@@ -6,10 +6,11 @@ use Lemonldap::NG::Common::Combination::Parser;
 use Lemonldap::NG::Portal::Main::Constants qw(PE_OK PE_ERROR);
 use Scalar::Util 'weaken';
 
-our $VERSION = '1.9.991_01';
+our $VERSION = '2.0.0';
 
 # TODO: See Lib::Wrapper
-extends 'Lemonldap::NG::Portal::Auth::Base';
+extends 'Lemonldap::NG::Portal::Main::Auth';
+with 'Lemonldap::NG::Portal::Lib::OverConf';
 
 # PROPERTIES
 
@@ -17,9 +18,10 @@ has stackSub => ( is => 'rw' );
 
 has wrapUserLogger => (
     is      => 'rw',
+    lazy    => 1,
     default => sub {
         Lemonldap::NG::Portal::Lib::Combination::UserLogger->new(
-            $_[0]->{userLogger} );
+            $_[0]->userLogger );
     }
 );
 
@@ -49,8 +51,8 @@ sub init {
 
         # "for" key can have 3 values:
         # 0: this module will be used for Auth and UserDB
-        # 1: this module will be user for Auth only
-        # 2: this module will be user for UserDB only
+        # 1: this module will be used for Auth only
+        # 2: this module will be used for UserDB only
 
         # Load Auth module
         if ( $mod->{for} < 2 ) {
@@ -59,6 +61,8 @@ sub init {
                 $self->error("Unable to load Auth::$mod->{type}");
                 return 0;
             }
+            $tmp[0]->{userLogger} = $self->wrapUserLogger;
+            weaken $tmp[0]->{userLogger};
         }
 
         # Load UserDB module
@@ -69,6 +73,8 @@ sub init {
                 $self->error("Unable to load UserDB::$mod->{type}");
                 return 0;
             }
+            $tmp[1]->{userLogger} = $self->wrapUserLogger;
+            weaken $tmp[1]->{userLogger};
         }
 
         # Store modules as array
@@ -97,7 +103,7 @@ sub extractFormInfo {
     return $self->try( 0, 'extractFormInfo', $req );
 }
 
-# Note that UserDB::Combination use the same object.
+# Note that UserDB::Combination uses the same object.
 sub getUser {
     return $_[0]->try( 1, 'getUser', $_[1] );
 }
@@ -123,8 +129,8 @@ sub getDisplayType {
     return $self->conf->{combinationForms}
       if ( $self->conf->{combinationForms} );
     my ( $nb, $stack ) = (
-        $req->datas->{dataKeep}->{combinationTry},
-        $req->datas->{combinationStack}
+        $req->data->{dataKeep}->{combinationTry},
+        $req->data->{combinationStack}
     );
     my ( $res, $name ) = $stack->[$nb]->[0]->( 'getDisplayType', $req );
     return $res;
@@ -134,7 +140,7 @@ sub authLogout {
     my ( $self, $req ) = @_;
     $self->getStack( $req, 'extractFormInfo' ) or return PE_ERROR;
     my ( $res, $name ) =
-      $req->datas->{combinationStack}->[ $req->userData->{_combinationTry} ]
+      $req->data->{combinationStack}->[ $req->userData->{_combinationTry} ]
       ->[0]->( 'authLogout', $req );
     $self->logger->debug(qq'User disconnected using scheme "$name"');
     return $res;
@@ -142,15 +148,14 @@ sub authLogout {
 
 sub getStack {
     my ( $self, $req, @steps ) = @_;
-    return $req->datas->{combinationStack}
-      if ( $req->datas->{combinationStack} );
-    my $stack = $req->datas->{combinationStack} =
-      $self->stackSub->( $req->env );
+    return $req->data->{combinationStack}
+      if ( $req->data->{combinationStack} );
+    my $stack = $req->data->{combinationStack} = $self->stackSub->( $req->env );
     unless ($stack) {
         $self->logger->error('No authentication scheme for this user');
     }
-    @{ $req->datas->{combinationSteps} } = ( @steps, @{ $req->steps } );
-    $req->datas->{dataKeep}->{combinationTry} ||= 0;
+    @{ $req->data->{combinationSteps} } = ( @steps, @{ $req->steps } );
+    $req->data->{dataKeep}->{combinationTry} ||= 0;
     return $stack;
 }
 
@@ -159,12 +164,12 @@ sub try {
     my ( $self, $type, $subname, $req ) = @_;
 
     # Get available authentication schemes for this user if not done
-    unless ( defined $req->datas->{combinationStack} ) {
+    unless ( defined $req->data->{combinationStack} ) {
         $self->getStack( $req, $subname ) or return PE_ERROR;
     }
     my ( $nb, $stack ) = (
-        $req->datas->{dataKeep}->{combinationTry},
-        $req->datas->{combinationStack}
+        $req->data->{dataKeep}->{combinationTry},
+        $req->data->{combinationStack}
     );
 
     # If more than 1 scheme is available
@@ -176,10 +181,10 @@ sub try {
 
         # On error, restart authentication with next scheme
         if ( $res > PE_OK ) {
-            $self->logger->info(
-                qq'Scheme "$name" has return $res, trying next');
-            $req->datas->{dataKeep}->{combinationTry}++;
-            $req->steps( [ @{ $req->datas->{combinationSteps} } ] );
+            $self->logger->info(qq'Scheme "$name" returned $res, trying next');
+            $req->data->{dataKeep}->{combinationTry}++;
+            $req->steps( [ @{ $req->data->{combinationSteps} } ] );
+            $req->continue(1);
             return PE_OK;
         }
     }
@@ -188,7 +193,7 @@ sub try {
     }
     $req->sessionInfo->{ [ '_auth', '_userDB' ]->[$type] } = $name;
     $req->sessionInfo->{_combinationTry} =
-      $req->datas->{dataKeep}->{combinationTry};
+      $req->data->{dataKeep}->{combinationTry};
     if ( $res > 0 ) {
         $self->userLogger->warn( 'All schemes failed'
               . ( $req->user ? ' for user ' . $req->user : '' ) );
@@ -205,30 +210,6 @@ sub name {
       || 'Combination';
 }
 
-# To avoid "tied" destroy, tied configurations are kept here
-our %overC;
-
-# Override portal loadPlugin() to use a wrapped configuration
-sub loadPlugin {
-    my ( $self, $plugin, $over ) = @_;
-    my $obj;
-    my $nc;
-    if ($over) {
-        require Lemonldap::NG::Common::Conf::Wrapper;
-        tie %$nc, 'Lemonldap::NG::Common::Conf::Wrapper', $self->conf, $over;
-        $overC{$plugin} = $nc;
-    }
-    else {
-        $nc = $self->conf;
-    }
-    return 0
-      unless ( $obj = $self->p->loadModule( "$plugin", $nc ) );
-    $obj = $self->p->findEP( $plugin, $obj );
-    $obj->{userLogger} = $self->wrapUserLogger;
-    weaken $obj->{userLogger};
-    return $obj;
-}
-
 package Lemonldap::NG::Portal::Lib::Combination::UserLogger;
 
 # This logger rewrite "warn" to "notice"
