@@ -4,9 +4,9 @@ use strict;
 use Mouse;
 use Lemonldap::NG::Portal::Main::Constants;
 
-our $VERSION = '1.9.991_01';
+our $VERSION = '2.0.0';
 
-extends 'Lemonldap::NG::Portal::Auth::Base';
+extends 'Lemonldap::NG::Portal::Main::Auth';
 
 # INITIALIZATION
 
