@@ -2,23 +2,20 @@ package Lemonldap::NG::Portal::Auth::SSL;
 
 use strict;
 use Mouse;
-use Lemonldap::NG::Portal::Main::Constants
-  qw(PE_OK PE_BADCERTIFICATE PE_CERTIFICATEREQUIRED);
+use Lemonldap::NG::Portal::Main::Constants qw(
+  PE_BADCERTIFICATE
+  PE_CERTIFICATEREQUIRED
+  PE_FIRSTACCESS
+  PE_OK
+);
 
-our $VERSION = '1.9.991_01';
+our $VERSION = '2.0.0';
 
-extends 'Lemonldap::NG::Portal::Auth::Base';
-
-# PROPERTIES
-
-has SSLField => ( is => 'rw' );
+extends 'Lemonldap::NG::Portal::Main::Auth';
 
 # INITIALIZATION
 
 sub init {
-    my ($self) = @_;
-    $self->SSLField( $self->conf->{SSLVar} ||= 'SSL_CLIENT_S_DN_Email' );
-    $self->conf->{SSLVarIf} ||= {};
     return 1;
 }
 
@@ -26,21 +23,31 @@ sub init {
 # @return Lemonldap::NG::Portal constant
 sub extractFormInfo {
     my ( $self, $req ) = @_;
-    my $field = $self->SSLField;
+    my $field = $self->conf->{SSLVar};
     if ( $req->env->{SSL_CLIENT_I_DN}
         and my $tmp =
         $self->conf->{SSLVarIf}->{ $req->env->{SSL_CLIENT_I_DN} } )
     {
         $field = $tmp;
     }
-    return PE_OK
-      if ( $req->user( $req->env->{$field} ) );
-    if ( $req->env->{SSL_CLIENT_S_DN} ) {
+    if ( $req->user( $req->env->{$field} ) ) {
+        $self->userLogger->notice( "GoodSSL authentication for " . $req->user );
+        return PE_OK;
+    }
+    elsif ( $req->env->{SSL_CLIENT_S_DN} ) {
         $self->userLogger->warn("$field was not found in user certificate");
         return PE_BADCERTIFICATE;
     }
+    elsif ( $self->conf->{sslByAjax} and not $req->param('nossl') ) {
+        $self->logger->debug('Send SSL javascript');
+        $req->data->{customScript} .=
+            '<script type="application/init">{"sslHost":"'
+          . $self->conf->{sslHost}
+          . '"}</script>';
+        return PE_FIRSTACCESS;
+    }
     else {
-        $self->userlogger->warn('No certificate found');
+        $self->userLogger->warn('No certificate found');
         return PE_CERTIFICATEREQUIRED;
     }
 }
@@ -56,7 +63,8 @@ sub setAuthSessionInfo {
 }
 
 sub getDisplayType {
-    return "logo";
+    my ($self) = @_;
+    return ( $self->{conf}->{sslByAjax} ? "sslform" : "logo" );
 }
 
 1;
