@@ -1,7 +1,7 @@
 package Lemonldap::NG::Portal::Issuer::OpenIDConnect;
 
 use strict;
-use JSON;
+use JSON qw(from_json to_json);
 use Mouse;
 use Lemonldap::NG::Common::FormEncode;
 use Lemonldap::NG::Portal::Main::Constants qw(
@@ -11,9 +11,10 @@ use Lemonldap::NG::Portal::Main::Constants qw(
   PE_LOGOUT_OK
   PE_REDIRECT
   PE_OK
+  PE_UNAUTHORIZEDPARTNER
 );
 
-our $VERSION = '1.9.991_01';
+our $VERSION = '2.0.0';
 
 extends 'Lemonldap::NG::Portal::Main::Issuer',
   'Lemonldap::NG::Portal::Lib::OpenIDConnect',
@@ -25,13 +26,18 @@ sub beforeAuth { 'exportRequestParameters' }
 
 # INITIALIZATION
 
+use constant sessionKind => 'OIDCI';
+
 has configStorage => (
     is      => 'ro',
+    lazy    => 1,
     default => sub {
         $_[0]->{p}->HANDLER->localConfig->{configStorage};
     }
 );
 
+has ssoMatchUrl => ( is => 'rw' );
+
 # OIDC has 7 endpoints managed here as PSGI endpoints or in run() [Main/Issuer.pm
 # manage transparent authentication for run()]:
 #  - authorize   : in run()
@@ -85,21 +91,38 @@ sub init {
         '.well-known' => { 'openid-configuration' => 'metadata' },
         ['GET']
     );
+    my $m =
+        '^/'
+      . $self->path . '/+(?:'
+      . join( '|',
+        $self->conf->{oidcServiceMetaDataAuthorizeURI},
+        $self->conf->{oidcServiceMetaDataEndSessionURI},
+      ) . ')';
+    $self->ssoMatchUrl(qr/$m/);
     return 1;
 }
 
 # RUNNING METHODS
 
+sub ssoMatch {
+    my ( $self, $req ) = @_;
+    return ( $req->uri =~ $self->ssoMatchUrl ? 1 : 0 );
+}
+
 # Main method (launched only for authenticated users, see Main/Issuer.pm)
 # run() manages only "authorize" and "logout" endpoints.
 sub run {
     my ( $self, $req, $path ) = @_;
     if ($path) {
 
+        # Convert old format OIDC Consents
+        my $ConvertedConsents = $self->_convertOldFormatConsents($req);
+        $self->logger->debug("$ConvertedConsents consent(s) converted");
+
         # AUTHORIZE
         if ( $path eq $self->conf->{oidcServiceMetaDataAuthorizeURI} ) {
             $self->logger->debug(
-                "URL  detected as an OpenID Connect AUTHORIZE URL");
+                "URL detected as an OpenID Connect AUTHORIZE URL");
 
             # Get and save parameters
             my $oidc_request = {};
@@ -113,6 +136,9 @@ sub run {
                     $oidc_request->{$param} = $req->param($param);
                     $self->logger->debug( "OIDC request parameter $param: "
                           . $oidc_request->{$param} );
+                    $self->p->setHiddenFormValue( $req, $param,
+                        $oidc_request->{$param},
+                        '', 0 );
                 }
             }
 
@@ -151,7 +177,8 @@ sub run {
 "Override $_ OIDC param by value present in request parameter"
                     );
                     $oidc_request->{$_} = $request->{$_};
-                    $self->p->setHiddenFormValue( $_, $request->{$_}, '' );
+                    $self->p->setHiddenFormValue( $req, $_, $request->{$_},
+                        '' );
                 }
             }
 
@@ -162,36 +189,60 @@ sub run {
             }
             unless ( $oidc_request->{'scope'} ) {
                 $self->logger->error("Scope is required");
-                $self->returnRedirectError(
-                    $req,
-                    $oidc_request->{'redirect_uri'},
-                    'invalid_request',
-                    "scope required",
-                    undef,
-                    $oidc_request->{'state'},
-                    ( $flow ne "authorizationcode" )
-                );
+                return PE_ERROR;
             }
             unless ( $oidc_request->{'client_id'} ) {
                 $self->logger->error("Client ID is required");
-                return $self->returnRedirectError(
-                    $req,
-                    $oidc_request->{'redirect_uri'},
-                    'invalid_request',
-                    "client_id required",
-                    undef,
-                    $oidc_request->{'state'},
-                    ( $flow ne "authorizationcode" )
-                );
+                return PE_ERROR;
             }
             if ( $flow eq "implicit" and not defined $oidc_request->{'nonce'} )
             {
                 $self->logger->error("Nonce is required for implicit flow");
-                return $self->returnRedirectError(
-                    $req, $oidc_request->{'redirect_uri'},
-                    'invalid_request', "nonce required",
-                    undef, $oidc_request->{'state'}, 1
+                return PE_ERROR;
+            }
+
+            # Check client_id
+            my $client_id = $oidc_request->{'client_id'};
+            $self->logger->debug("Request from client id $client_id");
+
+            # Verify that client_id is registered in configuration
+            my $rp = $self->getRP($client_id);
+
+            unless ($rp) {
+                $self->logger->error(
+"No registered Relying Party found with client_id $client_id"
                 );
+                return PE_UNAUTHORIZEDPARTNER;
+            }
+            else {
+                $self->logger->debug("Client id $client_id matches RP $rp");
+            }
+
+            # Check if this RP is authorized
+            if ( my $rule = $self->spRules->{$rp} ) {
+                unless ( $rule->( $req, $req->sessionInfo ) ) {
+                    $self->userLogger->warn( 'User '
+                          . $req->sessionInfo->{ $self->conf->{whatToTrace} }
+                          . "was not authorized to access to $rp" );
+                    return PE_UNAUTHORIZEDPARTNER;
+                }
+            }
+
+            # Check redirect_uri
+            my $redirect_uri  = $oidc_request->{'redirect_uri'};
+            my $redirect_uris = $self->conf->{oidcRPMetaDataOptions}->{$rp}
+              ->{oidcRPMetaDataOptionsRedirectUris};
+
+            if ($redirect_uris) {
+                my $redirect_uri_allowed = 0;
+                foreach ( split( /\s+/, $redirect_uris ) ) {
+                    $redirect_uri_allowed = 1 if $redirect_uri eq $_;
+                }
+                unless ($redirect_uri_allowed) {
+                    $self->userLogger->error(
+                        "Redirect URI $redirect_uri not allowed");
+                    return PE_BADURL;
+                }
             }
 
             # Check if flow is allowed
@@ -231,42 +282,28 @@ sub run {
             }
 
             # Check if user needs to be reauthenticated
-            my $reauthentication = 0;
-            my $prompt           = $oidc_request->{'prompt'};
-            if ( $prompt and $prompt =~ /\blogin\b/ ) {
+            my $prompt = $oidc_request->{'prompt'};
+            if (
+                    $prompt
+                and $prompt =~ /\blogin\b/
+                and (
+                    time - $req->sessionInfo->{_utime} >
+                    $self->conf->{portalForceAuthnInterval} )
+              )
+            {
                 $self->logger->debug(
-"Reauthentication requested by Relying Party in prompt parameter"
+"Reauthentication required by Relying Party in prompt parameter"
                 );
-                $reauthentication = 1;
+                return $self->reAuth($req);
             }
 
             my $max_age         = $oidc_request->{'max_age'};
             my $_lastAuthnUTime = $req->{sessionInfo}->{_lastAuthnUTime};
             if ( $max_age && time > $_lastAuthnUTime + $max_age ) {
                 $self->logger->debug(
-"Reauthentication forced cause authentication time ($_lastAuthnUTime) is too old (>$max_age s)"
+"Reauthentication forced because authentication time ($_lastAuthnUTime) is too old (>$max_age s)"
                 );
-                $reauthentication = 1;
-            }
-
-            if ($reauthentication) {
-
-                # Set prompt to 0 to avoid loop
-                $self->p->setHiddenFormValue( $req, 'prompt', '', '' );
-
-                # Replay authentication process
-                $self->{updateSession} = 1;
-                $req->steps(
-                    [
-                        $self->p->authProcess,
-                        @{ $self->p->betweenAuthAndDatas },
-                        $self->p->sessionDatas,
-                        @{ $self->p->afterDatas }
-                    ]
-                );
-
-                # Update session_id
-                return PE_OK;
+                return $self->reAuth($req);
             }
 
             # Check openid scope
@@ -277,31 +314,6 @@ sub run {
                 return PE_OK;
             }
 
-            # Check client_id
-            my $client_id = $oidc_request->{'client_id'};
-            $self->logger->debug("Request from client id $client_id");
-
-            # Verify that client_id is registered in configuration
-            my $rp = $self->getRP($client_id);
-
-            unless ($rp) {
-                $self->logger->error(
-"No registered Relying Party found with client_id $client_id"
-                );
-                return $self->returnRedirectError(
-                    $req,
-                    $oidc_request->{'redirect_uri'},
-                    'invalid_request',
-                    "client_id $client_id unknown",
-                    undef,
-                    $oidc_request->{'state'},
-                    ( $flow ne "authorizationcode" )
-                );
-            }
-            else {
-                $self->logger->debug("Client id $client_id match RP $rp");
-            }
-
             # Check Request JWT signature
             if ( $oidc_request->{'request'} ) {
                 unless (
@@ -312,28 +324,11 @@ sub run {
                   )
                 {
                     $self->logger->error(
-                        "Request JWT signature could not be verified");
+                        "JWT signature request can not be verified");
                     return PE_ERROR;
                 }
                 else {
-                    $self->logger->debug("Request JWT signature verified");
-                }
-            }
-
-            # Check redirect_uri
-            my $redirect_uri  = $oidc_request->{'redirect_uri'};
-            my $redirect_uris = $self->conf->{oidcRPMetaDataOptions}->{$rp}
-              ->{oidcRPMetaDataOptionsRedirectUris};
-
-            if ($redirect_uris) {
-                my $redirect_uri_allowed = 0;
-                foreach ( split( /\s+/, $redirect_uris ) ) {
-                    $redirect_uri_allowed = 1 if $redirect_uri eq $_;
-                }
-                unless ($redirect_uri_allowed) {
-                    $self->userLogger->error(
-                        "Redirect URI $redirect_uri not allowed");
-                    return PE_BADURL;
+                    $self->logger->debug("JWT signature request verified");
                 }
             }
 
@@ -352,12 +347,12 @@ sub run {
                 my $user_id = $req->{sessionInfo}->{$user_id_attribute};
                 unless ( $sub eq $user_id ) {
                     $self->userLogger->error(
-                        "ID Token hint sub $sub do not match user $user_id");
+                        "ID Token hint sub $sub does not match user $user_id");
                     return $self->returnRedirectError(
                         $req,
                         $oidc_request->{'redirect_uri'},
                         'invalid_request',
-                        "current user do not match id_token_hint sub",
+                        "Current user does not match id_token_hint sub",
                         undef,
                         $oidc_request->{'state'},
                         ( $flow ne "authorizationcode" )
@@ -365,7 +360,7 @@ sub run {
                 }
                 else {
                     $self->logger->debug(
-                        "ID Token hint sub $sub match current user");
+                        "ID Token hint sub $sub matches current user");
                 }
             }
 
@@ -374,20 +369,50 @@ sub run {
               ->{oidcRPMetaDataOptionsBypassConsent};
             if ($bypassConsent) {
                 $self->logger->debug(
-                    "Consent is disabled for RP $rp, user will not be prompted"
+"Consent is disabled for Relying Party $rp, user will not be prompted"
                 );
             }
             else {
                 my $ask_for_consent = 1;
-                if (    $req->{sessionInfo}->{"_oidc_consent_time_$rp"}
-                    and $req->{sessionInfo}->{"_oidc_consent_scope_$rp"} )
+                my $_oidcConsents;
+                my @RPoidcConsent = ();
+
+                # Loading existing oidcConsents
+                $self->logger->debug("Looking for OIDC Consents ...");
+
+                if ( $req->{sessionInfo}->{_oidcConsents} ) {
+                    $_oidcConsents = eval {
+                        from_json( $req->{sessionInfo}->{_oidcConsents},
+                            { allow_nonref => 1 } );
+                    };
+                    if ($@) {
+                        $self->logger->error(
+                            "Corrupted session (_oidcConsents): $@");
+                        return PE_ERROR;
+                    }
+                }
+                else {
+                    $self->logger->debug("No OIDC Consent found");
+                    $_oidcConsents = [];
+                }
+
+                # Read existing RP
+                @RPoidcConsent = grep { $_->{rp} eq $rp } @$_oidcConsents;
+                unless (@RPoidcConsent) {
+                    $self->logger->debug("No Relying Party $rp Consent found");
+
+                    # Set default value
+                    push @RPoidcConsent,
+                      { rp => $rp, epoch => '', scope => '' };
+                }
+
+                if (   $RPoidcConsent[0]{rp} eq $rp
+                    && $RPoidcConsent[0]{epoch}
+                    && $RPoidcConsent[0]{scope} )
                 {
                     $ask_for_consent = 0;
-                    my $consent_time =
-                      $req->{sessionInfo}->{"_oidc_consent_time_$rp"};
-                    my $consent_scope =
-                      $req->{sessionInfo}->{"_oidc_consent_scope_$rp"};
-
+                    my $consent_time  = $RPoidcConsent[0]{epoch};
+                    my $consent_scope = $RPoidcConsent[0]{scope};
                     $self->logger->debug(
 "Consent already given for Relying Party $rp (time: $consent_time, scope: $consent_scope)"
                     );
@@ -410,20 +435,21 @@ sub run {
                     }
 
                     # Check prompt parameter
-                    $ask_for_consent = 1 if ( $prompt =~ /\bconsent\b/ );
+                    $ask_for_consent = 1
+                      if ( $prompt and $prompt =~ /\bconsent\b/ );
                 }
                 if ($ask_for_consent) {
                     if (    $req->param('confirm')
                         and $req->param('confirm') == 1 )
                     {
-                        $self->p->updatePersistentSession(
-                            { "_oidc_consent_time_$rp" => time } );
-                        $self->p->updatePersistentSession(
-                            {
-                                "_oidc_consent_scope_$rp" =>
-                                  $oidc_request->{'scope'}
-                            }
-                        );
+                        $RPoidcConsent[0]{epoch} = time;
+                        $RPoidcConsent[0]{scope} = $oidc_request->{'scope'};
+                        push @{$_oidcConsents}, @RPoidcConsent;
+                        $self->logger->debug(
+                            "Append Relying Party $rp Consent");
+                        $self->p->updatePersistentSession( $req,
+                            { _oidcConsents => to_json($_oidcConsents) } );
+
                         $self->logger->debug(
                             "Consent given for Relying Party $rp");
                     }
@@ -444,12 +470,13 @@ sub run {
                     }
                     else {
                         $self->logger->debug(
-                            "Obtain user consent for Relying Party $rp");
+                            "Request user consent for Relying Party $rp");
 
                         # Return error if prompt is none
                         if ( $prompt and $prompt =~ /\bnone\b/ ) {
                             $self->logger->debug(
-                                "Consent is needed but prompt is none");
+                                "Consent is requiered but prompt is set to none"
+                            );
                             return $self->returnRedirectError(
                                 $req,
                                 $oidc_request->{'redirect_uri'},
@@ -466,23 +493,15 @@ sub run {
                           ->{oidcRPMetaDataOptionsDisplayName};
                         my $icon = $self->conf->{oidcRPMetaDataOptions}->{$rp}
                           ->{oidcRPMetaDataOptionsIcon};
-                        my $img_src;
+                        my $imgSrc;
 
                         if ($icon) {
-                            $img_src =
+                            $imgSrc =
                               ( $icon =~ m#^https?://# )
                               ? $icon
                               : $self->p->staticPrefic . "/common/" . $icon;
                         }
 
-                        # HERE
-                        $req->info('<div class="oidc_consent_message">');
-                        $req->info( '<img src="' . $img_src . '" />' )
-                          if $img_src;
-                        $req->info(
-qq'<h3 trspan="oidcConsent,$display_name">The application $display_name would like to know:</h3><ul>'
-                        );
-
                         my $scope_messages = {
                             openid  => 'yourIdentity',
                             profile => 'yourProfile',
@@ -490,16 +509,34 @@ qq'<h3 trspan="oidcConsent,$display_name">The application $display_name would li
                             address => 'yourAddress',
                             phone   => 'yourPhone',
                         };
+                        my @list;
                         foreach my $requested_scope (
                             split( /\s/, $oidc_request->{'scope'} ) )
                         {
-                            my $message = $scope_messages->{$requested_scope}
-                              || 'anotherInformation';
-                            $req->info(
-                                qq'<li trspan="$message ">$message</li>');
+                            if ( my $message =
+                                $scope_messages->{$requested_scope} )
+                            {
+                                push @list, { m => $message };
+                            }
+                            else {
+                                push @list,
+                                  {
+                                    m => 'anotherInformation',
+                                    n => $requested_scope
+                                  };
+                            }
                         }
-                        $req->info('</ul></div>');
-                        $req->datas->{activeTimer} = 0;
+                        $req->info(
+                            $self->loadTemplate(
+                                'oidcGiveConsent',
+                                params => {
+                                    displayName => $display_name,
+                                    imgUrl      => $imgSrc,
+                                    list        => \@list,
+                                }
+                            )
+                        );
+                        $req->data->{activeTimer} = 0;
                         return PE_CONFIRM;
                     }
                 }
@@ -773,7 +810,7 @@ qq'<h3 trspan="oidcConsent,$display_name">The application $display_name would li
                 return PE_REDIRECT;
             }
 
-            $self->logger->debug("No flow has been selected");
+            $self->logger->debug("None flow has been selected");
             return PE_OK;
         }
 
@@ -789,7 +826,7 @@ qq'<h3 trspan="oidcConsent,$display_name">The application $display_name would li
                 if ( $oidc_request->{$param} = $req->param($param) ) {
                     $self->logger->debug( "OIDC request parameter $param: "
                           . $oidc_request->{$param} );
-                    $self->p->setHiddenFormValue( $param,
+                    $self->p->setHiddenFormValue( $req, $param,
                         $oidc_request->{$param}, '' );
                 }
             }
@@ -800,6 +837,7 @@ qq'<h3 trspan="oidcConsent,$display_name">The application $display_name would li
 
             # Ask consent for logout
             if ( $req->param('confirm') ) {
+                my $err;
                 if ( $req->param('confirm') == 1 ) {
                     $req->steps(
                         [
@@ -807,16 +845,45 @@ qq'<h3 trspan="oidcConsent,$display_name">The application $display_name would li
                             'deleteSession'
                         ]
                     );
-                    my $err = $req->error( $self->p->process($req) );
-                    if ( $err > 0 ) {
-                        $self->logger->error(
-                            "Logout process returns error code $err");
-                        return PE_ERROR;
+                    $err = $req->error( $self->p->process($req) );
+                    if ( $err and $err != PE_LOGOUT_OK ) {
+                        if ( $err > 0 ) {
+                            $self->logger->error(
+                                "Logout process returns error code $err");
+                            return PE_ERROR;
+                        }
+                        return $err;
                     }
                 }
 
                 if ($post_logout_redirect_uri) {
 
+                    # Check redirect URI is allowed
+                    my $redirect_uri_allowed = 0;
+                    foreach ( keys %{ $self->conf->{oidcRPMetaDataOptions} } ) {
+                        my $logout_rp = $_;
+                        if ( my $redirect_uris =
+                            $self->conf->{oidcRPMetaDataOptions}->{$logout_rp}
+                            ->{oidcRPMetaDataOptionsPostLogoutRedirectUris} )
+                        {
+
+                            foreach ( split( /\s+/, $redirect_uris ) ) {
+                                if ( $post_logout_redirect_uri eq $_ ) {
+                                    $self->logger->debug(
+"$post_logout_redirect_uri is an allowed logout redirect URI for RP $logout_rp"
+                                    );
+                                    $redirect_uri_allowed = 1;
+                                }
+                            }
+                        }
+                    }
+
+                    unless ($redirect_uri_allowed) {
+                        $self->logger->error(
+                            "$post_logout_redirect_uri is not allowed");
+                        return PE_BADURL;
+                    }
+
                     # Build Response
                     my $response_url =
                       $self->buildLogoutResponse( $post_logout_redirect_uri,
@@ -826,13 +893,13 @@ qq'<h3 trspan="oidcConsent,$display_name">The application $display_name would li
                     $req->urldc($response_url);
                     return PE_REDIRECT;
                 }
-                return $req->param('confirm') == 1 ? PE_LOGOUT_OK : PE_OK;
+                return $req->param('confirm') == 1
+                  ? ( $err ? $err : PE_LOGOUT_OK )
+                  : PE_OK;
             }
 
-            $req->info(
-'<div class="oidc_logout_message"><h3 trspan="logoutConfirm">Do you want to logout?</h3></div>'
-            );
-            $req->datas->{activeTimer} = 0;
+            $req->info( $self->loadTemplate('oidcLogout') );
+            $req->data->{activeTimer} = 0;
             return PE_CONFIRM;
         }
     }
@@ -865,7 +932,7 @@ sub token {
         return $self->p->sendError( $req, 'invalid_request', 400 );
     }
     else {
-        $self->logger->debug("Client id $client_id match RP $rp");
+        $self->logger->debug("Client id $client_id match Relying Party $rp");
     }
 
     # Check client_secret
@@ -885,15 +952,15 @@ sub token {
 
     unless ($codeSession) {
         $self->logger->error("Unable to find OIDC session $code");
-        $self->p->sendError( $req, 'invalid_request', 400 );
+        return $self->p->sendError( $req, 'invalid_request', 400 );
     }
 
     # Check we have the same redirect_uri value
     unless ( $req->param("redirect_uri") eq $codeSession->data->{redirect_uri} )
     {
-        $self->userLogger->error( "Provided redirect_uri is different from "
+        $self->userLogger->error( "Provided redirect_uri does not match "
               . $codeSession->{redirect_uri} );
-        $self->p->sendError( $req, 'invalid_request', 400 );
+        return $self->p->sendError( $req, 'invalid_request', 400 );
     }
 
     # Get user identifier
@@ -905,7 +972,7 @@ sub token {
         $self->userLogger->error(
             "Unable to find user session linked to OIDC session $code");
         $codeSession->remove();
-        $self->p->sendError( $req, 'invalid_request', 400 );
+        return $self->p->sendError( $req, 'invalid_request', 400 );
     }
 
     my $user_id_attribute =
@@ -931,7 +998,7 @@ sub token {
         $self->userLogger->error(
             "Unable to create OIDC session for access_token");
         $codeSession->remove();
-        $self->p->sendError( $req, 'invalid_request', 400 );
+        return $self->p->sendError( $req, 'invalid_request', 400 );
     }
 
     my $access_token = $accessTokenSession->id;
@@ -984,9 +1051,11 @@ sub token {
         id_token     => $id_token,
     };
 
-    my $cRP = join ',', $rp, $apacheSession->data->{_oidcConnectedRP} || '';
-    $self->p->updateSession( $req, { _oidcConnectedRP => $cRP },
-        $apacheSession->id );
+    my $cRP = $apacheSession->data->{_oidcConnectedRP} || '';
+    unless ( $cRP =~ /\b$rp\b/ ) {
+        $self->p->updateSession( $req, { _oidcConnectedRP => "$rp,$cRP" },
+            $apacheSession->id );
+    }
 
     $self->logger->debug("Send token response");
 
@@ -1086,13 +1155,13 @@ sub registration {
     # Check dynamic registration is allowed
     unless ( $self->conf->{oidcServiceAllowDynamicRegistration} ) {
         $self->logger->error("Dynamic registration is not allowed");
-        $self->p->sendError( $req, 'server_error' );
+        return $self->p->sendError( $req, 'server_error' );
     }
 
     # Get client metadata
     my $client_metadata_json = $req->content;
     unless ($client_metadata_json) {
-        return $self->p->sendError( $req, 'Missing POST datas', 400 );
+        return $self->p->sendError( $req, 'Missing POST data', 400 );
     }
 
     $self->logger->debug("Client metadata received: $client_metadata_json");
@@ -1189,6 +1258,29 @@ sub endSessionDone {
 
     if ($post_logout_redirect_uri) {
 
+        # Check redirect URI is allowed
+        my $redirect_uri_allowed = 0;
+        foreach ( keys %{ $self->conf->{oidcRPMetaDataOptions} } ) {
+            my $logout_rp = $_;
+            my $redirect_uris =
+              $self->conf->{oidcRPMetaDataOptions}->{$logout_rp}
+              ->{oidcRPMetaDataOptionsPostLogoutRedirectUris};
+
+            foreach ( split( /\s+/, $redirect_uris ) ) {
+                if ( $post_logout_redirect_uri eq $_ ) {
+                    $self->logger->debug(
+"$post_logout_redirect_uri is an allowed logout redirect URI for RP $logout_rp"
+                    );
+                    $redirect_uri_allowed = 1;
+                }
+            }
+        }
+
+        unless ($redirect_uri_allowed) {
+            $self->logger->error("$post_logout_redirect_uri is not allowed");
+            return $self->p->login($req);
+        }
+
         # Build Response
         my $response_url =
           $self->buildLogoutResponse( $post_logout_redirect_uri, $state );
@@ -1204,8 +1296,7 @@ sub endSessionDone {
 # Handle checksession endpoint
 sub checkSession {
     my ( $self, $req ) = @_;
-    $self->logger->debug(
-        "URL  detected as an OpenID Connect CHECK SESSION URL");
+    $self->logger->debug("URL detected as an OpenID Connect CHECK SESSION URL");
 
     # TODO: access_control_allow_origin => '*'
     $req->frame(1);
@@ -1232,7 +1323,7 @@ sub logout {
         foreach my $rp (@rps) {
             my $rpConf = $self->conf->{oidcRPMetaDataOptions}->{$rp};
             unless ($rpConf) {
-                $self->logger->error("Unknown RP $rp");
+                $self->logger->error("Unknown Relying Party $rp");
                 return PE_ERROR;
             }
             if ( my $url = $rpConf->{oidcRPMetaDataOptionsLogoutUrl} ) {
@@ -1244,9 +1335,10 @@ sub logout {
                           ->{oidcRPMetaDataOptionsUserIDAttr}
                           || $self->conf->{whatToTrace};
                         my $user_id = $req->{sessionInfo}->{$user_id_attribute};
+                        my $iss     = $self->conf->{oidcServiceMetaDataIssuer};
                         $url .= ( $url =~ /\?/ ? '&' : '?' )
                           . build_urlencoded(
-                            iss => $self->conf->{oidcServiceMetaDataIssuer},
+                            iss => $iss,
                             sid => $user_id
                           );
                     }
@@ -1397,4 +1489,60 @@ sub exportRequestParameters {
     return PE_OK;
 }
 
+sub _convertOldFormatConsents {
+    my ( $self, $req ) = @_;
+    my @oidcConsents = ();
+    my @rps          = ();
+    my $scope        = '';
+    my $epoch        = '';
+    my $rp           = '';
+    unless ( $req->{sessionInfo} ) {
+        $self->logger->error("Corrupted session");
+        return PE_ERROR;
+    }
+
+    # Search Relying Parties
+    $self->logger->debug(
+        "Searching for previously registered Relying Parties...");
+    foreach ( keys %{ $req->{sessionInfo} } ) {
+        if ( $_ =~ /^_oidc_consent_scope_([\w-]+)$/ ) {
+            push @rps, $1;
+            $self->logger->debug("Found RP $1");
+        }
+    }
+
+    # Convert OIDC Consents format
+    $self->logger->debug("Convert Relying Party Consent(s)...");
+    my $count = 0;
+    foreach (@rps) {
+        $scope = $req->{sessionInfo}->{ "_oidc_consent_scope_" . $_ };
+        $epoch = $req->{sessionInfo}->{ "_oidc_consent_time_" . $_ };
+        $rp    = $_;
+
+        if ( $scope and $epoch and $rp ) {
+            $self->logger->debug("Append RP $rp Consent");
+            push @oidcConsents, { rp => $rp, scope => $scope, epoch => $epoch };
+            $count++;
+            $self->logger->debug("Delete Key -> _oidc_consent_scope_$_");
+            $self->p->updatePersistentSession( $req,
+                { "_oidc_consent_scope_" . $_ => undef } );
+            $self->logger->debug("Delete Key -> _oidc_consent_time_$_");
+            $self->p->updatePersistentSession( $req,
+                { "_oidc_consent_time_" . $_ => undef } );
+        }
+        else {
+            $self->logger->debug(
+"Corrupted Consent / Session -> RP=$rp, Scope=$scope, Epoch=$epoch"
+            );
+            return PE_ERROR;
+        }
+    }
+
+    # Update persistent session
+    $self->p->updatePersistentSession( $req,
+        { _oidcConsents => to_json( \@oidcConsents ) } )
+      if $count;
+    return $count;
+}
+
 1;
