@@ -2,6 +2,7 @@ package Lemonldap::NG::Portal::Issuer::SAML;
 
 use strict;
 use Mouse;
+use Lemonldap::NG::Portal::Lib::SAML;
 use Lemonldap::NG::Portal::Main::Constants qw(
   PE_OK
   PE_SAML_ART_ERROR
@@ -11,9 +12,10 @@ use Lemonldap::NG::Portal::Main::Constants qw(
   PE_SAML_SLO_ERROR
   PE_SAML_SSO_ERROR
   PE_SAML_UNKNOWN_ENTITY
+  PE_UNAUTHORIZEDPARTNER
 );
 
-our $VERSION = '1.9.991_01';
+our $VERSION = '2.0.0';
 
 extends 'Lemonldap::NG::Portal::Main::Issuer',
   'Lemonldap::NG::Portal::Lib::SAML';
@@ -22,6 +24,11 @@ has ssoUrlRe => ( is => 'rw' );
 
 has ssoUrlArtifact => ( is => 'rw' );
 
+has ssoGetUrl => ( is => 'rw' );
+use constant sessionKind => 'ISAML';
+use constant lsDump      => '_lassoSessionDumpI';
+use constant liDump      => '_lassoIdentityDumpI';
+
 # INTERFACE
 
 # Simply store SP in $req->env
@@ -33,8 +40,11 @@ sub init {
     my ($self) = @_;
 
     # Prepare SSO URL catching
-    my $saml_sso_get_url = $self->getMetaDataURL(
-        "samlIDPSSODescriptorSingleSignOnServiceHTTPRedirect", 1 );
+    my $saml_sso_get_url = $self->ssoGetUrl(
+        $self->getMetaDataURL(
+            "samlIDPSSODescriptorSingleSignOnServiceHTTPRedirect", 1
+        )
+    );
     my $saml_sso_get_url_ret = $self->getMetaDataURL(
         "samlIDPSSODescriptorSingleSignOnServiceHTTPRedirect", 2 );
     my $saml_sso_post_url =
@@ -67,6 +77,7 @@ qr/^($saml_sso_get_url|$saml_sso_get_url_ret|$saml_sso_post_url|$saml_sso_post_u
           # Required to manage SLO in Proxy mode
           and $self->loadIDPs()
     );
+    return 0 unless ($res);
 
     # Single logout routes
     $self->addUnauthRouteFromMetaDataURL(
@@ -151,6 +162,17 @@ sub storeEnv {
     return PE_OK;
 }
 
+sub ssoMatch {
+    my ( $self, $req ) = @_;
+    my $url = $self->normalize_url( $req->uri, $self->conf->{issuerDBSAMLPath},
+        $self->ssoGetUrl );
+    return (
+        $url =~ $self->ssoUrlRe or $req->data->{_proxiedRequest}
+        ? 1
+        : 0
+    );
+}
+
 # Main method (launched only for authenticated users, see Main/Issuer)
 sub run {
     my ( $self, $req ) = @_;
@@ -174,8 +196,19 @@ sub run {
     my $idp_initiated_sp        = $req->param('sp');
     my $idp_initiated_spConfKey = $req->param('spConfKey');
 
+    # Normalize URL to be tolerant to SAML Path
+    $url = $self->normalize_url( $url, $self->conf->{issuerDBSAMLPath},
+        $self->ssoGetUrl );
+
+    # Get domain GET attribute
+    my $domain = $req->param('domain');
+
+    if ($domain) {
+        $self->logger->debug("Found domain $domain in SAML GET parameter");
+    }
+
     # 1.1. SSO (SSO URL or Proxy Mode)
-    if ( $url =~ $self->ssoUrlRe or $req->datas->{_proxiedRequest} ) {
+    if ( $url =~ $self->ssoUrlRe or $req->data->{_proxiedRequest} ) {
 
         $self->logger->debug("URL $url detected as an SSO request URL");
 
@@ -189,12 +222,20 @@ sub run {
         # Ignore signature verification
         $self->disableSignatureVerification($login);
 
+        if ($request) {
+            $req->data->{_proxiedSamlRequest} = $login->request();
+            $req->data->{_proxiedRequest}     = $request;
+            $req->data->{_proxiedMethod}      = $method;
+            $req->data->{_proxiedRelayState}  = $relaystate,
+              $req->data->{_proxiedArtifact}  = $artifact;
+        }
+
         # Process the request or use IDP initiated mode
         if ( $request or $idp_initiated ) {
 
             # Load Session and Identity if they exist
-            my $session  = $req->{sessionInfo}->{_lassoSessionDump};
-            my $identity = $req->{sessionInfo}->{_lassoIdentityDump};
+            my $session  = $req->{sessionInfo}->{ $self->lsDump };
+            my $identity = $req->{sessionInfo}->{ $self->liDump };
 
             if ($session) {
                 unless ( $self->setSessionFromDump( $login, $session ) ) {
@@ -314,6 +355,15 @@ sub run {
             $self->logger->debug("$sp match $spConfKey SP in configuration");
             $req->env->{llng_saml_spconfkey} = $spConfKey;
 
+            if ( my $rule = $self->spRules->{$sp} ) {
+                unless ( $rule->( $req, $req->sessionInfo ) ) {
+                    $self->userLogger->warn( 'User '
+                          . $req->sessionInfo->{ $self->conf->{whatToTrace} }
+                          . "was not authorizated to access to $sp" );
+                    return PE_UNAUTHORIZEDPARTNER;
+                }
+            }
+
             # Do we check signature?
             my $checkSSOMessageSignature =
               $self->conf->{samlSPMetaDataOptions}->{$spConfKey}
@@ -342,6 +392,18 @@ sub run {
                 $self->logger->debug("Message signature will not be checked");
             }
 
+            # Force AllowCreate to TRUE for transient/persistent NameIDPolicy
+            if ( $login->request()->NameIDPolicy ) {
+                my $nif = $login->request()->NameIDPolicy->Format();
+                if (   $nif eq $self->getNameIDFormat("transient")
+                    or $nif eq $self->getNameIDFormat("persistent") )
+                {
+                    $self->logger->debug(
+                        "Force AllowCreate flag in NameIDPolicy");
+                    eval { $login->request()->NameIDPolicy()->AllowCreate(1); };
+                }
+            }
+
             # Validate request
             unless ( $self->validateRequestMsg( $login, 1, 1 ) ) {
                 $self->logger->error("Unable to validate SSO request message");
@@ -363,106 +425,25 @@ sub run {
             $self->logger->debug(
                 "Found ForceAuthn flag with value $force_authn");
 
-            # Get ForceAuthn sessions for this session_id
-            my $moduleOptions = $self->conf->{samlStorageOptions} || {};
-            $moduleOptions->{backend} = $self->conf->{samlStorage};
-            my $module = "Lemonldap::NG::Common::Apache::Session";
-
-            my $forceAuthn_sessions =
-              $module->searchOn( $moduleOptions, "_saml_id", $session_id );
-
-            my $forceAuthn_session;
-            my $forceAuthnSessionInfo;
-
+            # Force authentication if flag is on, or previous flag still active
             if (
-                my @forceAuthn_sessions_keys =
-                keys %$forceAuthn_sessions
+                $force_authn
+                and (
+                    time - $req->sessionInfo->{_utime} >
+                    $self->conf->{portalForceAuthnInterval} )
               )
             {
 
-                # Warning if more than one session found
-                if ( $#forceAuthn_sessions_keys > 0 ) {
-                    $self->logger->warn(
-"More than one ForceAuthn session found for session $session_id"
-                    );
-                }
-
-                # Take the first session
-                $forceAuthn_session = shift @forceAuthn_sessions_keys;
-
-                # Get session
-                $self->logger->debug(
-"Retrieve ForceAuthn session $forceAuthn_session for session $session_id"
-                );
-
-                $forceAuthnSessionInfo =
-                  $self->getSamlSession($forceAuthn_session);
-
-                # Check forceAuthn flag for current SP
-                if ( $forceAuthnSessionInfo->data->{$spConfKey} ) {
-
-                    $self->logger->debug(
-"User was already forced to reauthenticate for SP $spConfKey"
-                    );
-                    $force_authn = 1;
-                }
-            }
-            else {
-                $self->logger->debug(
-                    "No ForceAuthn session found for session $session_id");
-            }
-
-            # Force authentication if flag is on, or previous flag still active
-            if ($force_authn) {
-                my $info = { $spConfKey => 1 };
-
-                unless ($forceAuthn_session) {
-                    my $forceInfos;
-                    $info->{'_type'}    = "forceAuthn";
-                    $info->{'_saml_id'} = $session_id;
-                    $info->{'_utime'}   = $time;
-                    $self->logger->debug(
-                        "Create ForceAuthn session $forceAuthn_session");
-                }
-
-                # Store flag for further requests
-                $forceAuthnSessionInfo =
-                  $self->getSamlSession( $forceAuthn_session, $info );
-                $forceAuthn_session = $forceAuthnSessionInfo->id
-                  unless ($forceAuthn_session);
-
-                $self->logger->debug(
-"Set ForceAuthn flag for SP $spConfKey in ForceAuthn session $forceAuthn_session"
-                );
+                $self->userLogger->info(
+                    "SAML SP $sp ask to refresh session of "
+                      . $req->sessionInfo->{ $self->conf->{whatToTrace} } );
 
                 # Replay authentication process
-                $req->{updateSession} = 1;
-                $req->steps(
-                    [
-                        @{ $self->p->beforeAuth },
-                        $self->p->authProcess,
-                        @{ $self->p->betweenAuthAndDatas },
-                        $self->p->sessionDatas,
-                        @{ $self->p->afterDatas }
-                    ]
-                );
-                $req->error( $self->p->process($req) );
-
-                # Return error if any
-                return $req->{error} if $req->{error} > 0;
-
-                # Else remove flag
-                $forceAuthnSessionInfo =
-                  $self->getSamlSession( $forceAuthn_session,
-                    { $spConfKey => 0 } );
-
-                $self->logger->debug(
-"Unset ForceAuthn flag for SP $spConfKey in ForceAuthn session $forceAuthn_session"
-                );
+                return $self->reAuth($req);
             }
 
             # Check Destination (only in non proxy mode)
-            unless ( $req->datas->{_proxiedRequest} ) {
+            unless ( $req->data->{_proxiedRequest} ) {
                 return PE_SAML_DESTINATION_ERROR
                   unless ( $self->checkDestination( $login->request, $url ) );
             }
@@ -664,6 +645,16 @@ sub run {
                 return PE_SAML_SSO_ERROR;
             }
 
+            # Rewrite Issuer with domain
+            if ($domain) {
+                my $original_issuer = $login->response->Issuer->content;
+                $self->logger->debug(
+                    "Add domain $domain to Issuer $original_issuer");
+                my $new_issuer = $original_issuer . "?domain=$domain";
+                $login->response->Issuer->content($new_issuer);
+                $login->response->Assertion->Issuer->content($new_issuer);
+            }
+
             # Set subject NameID
             $response_assertions[0]
               ->set_subject_name_id( $login->nameIdentifier );
@@ -709,12 +700,18 @@ sub run {
             my @authn_statements = $response_assertions[0]->AuthnStatement();
 
             # Set sessionIndex
-            # sessionIndex is the encrypted session_id
-            my $sessionIndex = $self->conf->{cipher}->encrypt($session_id);
+            my $sessionIndexSession = $self->getSamlSession();
+            return PE_SAML_SESSION_ERROR unless $sessionIndexSession;
+
+            $sessionIndexSession->update(
+                { '_utime' => time, '_saml_id' => $session_id } );
+            my $sessionIndex = $sessionIndexSession->id;
+
             $authn_statements[0]->SessionIndex($sessionIndex);
 
             $self->logger->debug(
-                "Set sessionIndex $sessionIndex (encrypted from $session_id)");
+                "Set sessionIndex $sessionIndex (linked to session $session_id)"
+            );
 
             # Set SessionNotOnOrAfter
             my $sessionNotOnOrAfterTimeout =
@@ -773,7 +770,6 @@ sub run {
 
             # Artifact
             # Choose method
-            no strict 'subs';
             if (   $artifact
                 or $protocolProfile ==
                 Lasso::Constants::LOGIN_PROTOCOL_PROFILE_BRWS_ART )
@@ -782,7 +778,7 @@ sub run {
                 if (   $method == $self->getHttpMethod("post")
                     || $method == $self->getHttpMethod("artifact-post") )
                 {
-                    $artifact_method = $self->getHttpMethod("artifact-post")
+                    $artifact_method = $self->getHttpMethod("artifact-post");
 
                 }
                 else {
@@ -790,7 +786,6 @@ sub run {
                 }
             }
 
-            no strict 'subs';
             if ( $protocolProfile ==
                 Lasso::Constants::LOGIN_PROTOCOL_PROFILE_BRWS_ART )
             {
@@ -830,15 +825,15 @@ sub run {
 
                 # Update session
                 $self->logger->debug("Save Lasso identity in session");
-                $self->updatePersistentSession(
-                    { _lassoIdentityDump => $login->get_identity->dump },
+                $self->p->updatePersistentSession( $req,
+                    { $self->liDump => $login->get_identity->dump },
                     undef, $session_id );
             }
 
             if ( $login->is_session_dirty ) {
                 $self->logger->debug("Save Lasso session in session");
                 $self->p->updateSession( $req,
-                    { _lassoSessionDump => $login->get_session->dump },
+                    { $self->lsDump => $login->get_session->dump },
                     $session_id );
             }
 
@@ -893,8 +888,10 @@ sub run {
                   . ' width="0" height="0" frameborder="0"></iframe>';
 
                 $req->info(
-                    '<h3 trspan="updateCdc">Update Common Domain Cookie</h3>'
-                      . $cdc_iframe );
+                    $self->loadTemplate( 'simpleInfo',
+                        params => { trspan => 'updateCdc' } )
+                      . $cdc_iframe
+                );
             }
 
             # HTTP-POST
@@ -967,13 +964,15 @@ sub run {
         }
 
     }
+    $self->logger->debug("Not an issuer request $url");
     return PE_OK;
 }
 
 sub artifactServer {
     my ( $self, $req ) = @_;
-    $self->logger->debug(
-        "URL $req->uri detected as an artifact resolution service URL");
+    $self->logger->debug( "URL "
+          . $req->uri
+          . " detected as an artifact resolution service URL" );
 
     # Artifact request are sent with SOAP trough POST
     my $art_request = $req->content;
@@ -1098,17 +1097,24 @@ sub soapSloServer {
         my $session_index;
         eval { $session_index = $logout->request()->SessionIndex; };
 
-        # SLO requests without session index are not accepted
+        # SLO requests without session index are not accepted in SOAP mode
         unless ( defined $session_index ) {
             $self->p->sendError( $req,
                 "No session index in SLO request from $spConfKey SP", 400 );
         }
 
-        # Decrypt session index
-        my $local_session_id = $self->conf->{cipher}->decrypt($session_index);
+        # Get session index
+        my $sessionIndexSession = $self->getSamlSession($session_index);
+        return $self->p->sendError( $req, 'SAML session not found', 400 )
+          unless $sessionIndexSession;
+
+        my $local_session_id = $sessionIndexSession->data->{_saml_id};
+
+        $sessionIndexSession->remove;
 
         $self->logger->debug(
-            "Get session id $local_session_id (decrypted from $session_index)");
+"Get session id $local_session_id (from session index $session_index)"
+        );
 
         # Open local session
         my $local_session = $self->p->getApacheSession($local_session_id);
@@ -1118,8 +1124,8 @@ sub soapSloServer {
         }
 
         # Load Session and Identity if they exist
-        my $session  = $local_session->data->{_lassoSessionDump};
-        my $identity = $local_session->data->{_lassoIdentityDump};
+        my $session  = $local_session->data->{ $self->lsDump };
+        my $identity = $local_session->data->{ $self->liDump };
 
         if ($session) {
             unless ( $self->setSessionFromDump( $logout, $session ) ) {
@@ -1215,7 +1221,7 @@ sub soapSloServer {
 
 sub logout {
     my ( $self, $req ) = @_;
-    return PE_OK if ( $req->datas->{samlSLOCalled} );
+    return PE_OK if ( $req->data->{samlSLOCalled} );
 
     # Session ID
     my $session_id = $req->{sessionInfo}->{_session_id} || $req->{id};
@@ -1229,8 +1235,8 @@ sub logout {
     my $logout = $self->createLogout( $self->lassoServer );
 
     # Load Session and Identity if they exist
-    my $session  = $req->{sessionInfo}->{_lassoSessionDump};
-    my $identity = $req->{sessionInfo}->{_lassoIdentityDump};
+    my $session  = $req->{sessionInfo}->{ $self->lsDump };
+    my $identity = $req->{sessionInfo}->{ $self->liDump };
 
     if ($session) {
         unless ( $self->setSessionFromDump( $logout, $session ) ) {
@@ -1297,8 +1303,8 @@ sub sloRelaySoap {
     }
 
     # Load Session and Identity if they exist
-    my $session    = $relayInfos->data->{_lassoSessionDump};
-    my $identity   = $relayInfos->data->{_lassoIdentityDump};
+    my $session    = $relayInfos->data->{ $self->lsDump };
+    my $identity   = $relayInfos->data->{ $self->liDump };
     my $providerID = $relayInfos->data->{_providerID};
     my $relayState = $relayInfos->data->{_relayState} // '';
     my $spConfKey  = $self->spList->{$providerID}->{confKey};
@@ -1320,7 +1326,6 @@ sub sloRelaySoap {
     }
 
     # Send the logout request
-    no strict 'subs';
     my ( $rstatus, $rmethod, $rinfo ) =
       $self->sendLogoutRequestToProvider( $req, $logout, $providerID,
         Lasso::Constants::HTTP_METHOD_SOAP,
@@ -1385,7 +1390,7 @@ sub sloRelayPost {
 
 sub authSloServer {
     my ( $self, $req ) = @_;
-    $self->p->importHandlerDatas($req);
+    $self->p->importHandlerData($req);
     return $self->sloServer($req);
 }
 
@@ -1443,19 +1448,33 @@ sub sloServer {
 
         eval { $session_index = $logout->request()->SessionIndex; };
 
-        # SLO requests without session index are not accepted
+        # SLO requests without session index can be accepted
         unless ( defined $session_index ) {
-            $self->logger->error(
+            $self->logger->warn(
                 "No session index in SLO request from $spConfKey SP");
-            return $self->sendSLOErrorResponse( $logout, $method );
         }
-        $local_session_id = $self->conf->{cipher}->decrypt($session_index);
-        $self->logger->debug(
-            "Get session id $local_session_id (decrypted from $session_index)");
 
+        if ($session_index) {
+            my $sessionIndexSession = $self->getSamlSession($session_index);
+            return $self->p->sendError( $req, 'SAML session not found', 400 )
+              unless $sessionIndexSession;
+
+            $local_session_id = $sessionIndexSession->data->{_saml_id};
+
+            $sessionIndexSession->remove;
+
+            $self->logger->debug(
+"Get session id $local_session_id (from session index $session_index)"
+            );
+        }
+        else {
+            $local_session_id = $req->id;
+            $self->logger->debug(
+                "Get session id $local_session_id (from cookie)");
+        }
         if ( $req->{sessionInfo} ) {
-            $session  = $req->{sessionInfo}->{_lassoSessionDump};
-            $identity = $req->{sessionInfo}->{_lassoIdentityDump};
+            $session  = $req->{sessionInfo}->{ $self->lsDump };
+            $identity = $req->{sessionInfo}->{ $self->liDump };
         }
         unless ($session) {
 
@@ -1468,10 +1487,10 @@ sub sloServer {
             }
 
             # Load Session and Identity if they exist
-            $session  = $local_session->data->{_lassoSessionDump};
-            $identity = $local_session->data->{_lassoIdentityDump};
+            $session  = $local_session->data->{ $self->lsDump };
+            $identity = $local_session->data->{ $self->liDump };
 
-            # Import user datas in $req (for other "logout" subs)
+            # Import user data in $req (for other "logout" subs)
             $req->id( $local_session->data->{_session_id} );
             $req->sessionInfo( $local_session->data );
             $req->user( $local_session->data->{ $self->conf->{whatToTrace} } );
@@ -1552,7 +1571,7 @@ sub sloServer {
 
         # Close local session
         # This flag is for logout() to say that SAML logout is already done
-        $req->datas->{samlSLOCalled} = 1;
+        $req->data->{samlSLOCalled} = 1;
 
         # Launch normal logout and ignore errors
         $self->p->do( $req, [ @{ $self->p->beforeLogout }, 'deleteSession' ] );
@@ -1577,7 +1596,7 @@ sub sloServer {
         else {
             $req->{urldc} =
               $self->conf->{portal} . '/saml/relaySingleLogoutTermination';
-            $self->p->setHiddenFormValue( 'relay', $relayID );
+            $self->p->setHiddenFormValue( $req, 'relay', $relayID );
             return $self->do( $req, [] );
         }
 
@@ -1589,7 +1608,7 @@ sub sloServer {
         # else it means SSO session was not closed: launching it
         $self->logger->debug(
             "SLO response found on an active SSO session, ignoring it");
-        $req->datas->{samlSLOCalled} = 1;
+        $req->data->{samlSLOCalled} = 1;
         return $self->p->do( $req,
             [ @{ $self->p->beforeLogout }, 'deleteSession' ] );
     }
@@ -1656,14 +1675,17 @@ sub attributeServer {
 
     # Get sessionInfo for the given NameID
     my $sessionInfo;
-    my $moduleOptions = $self->conf->{samlStorageOptions} || {};
-    $moduleOptions->{backend} = $self->conf->{samlStorage};
-    my $module = "Lemonldap::NG::Common::Apache::Session";
 
     my $saml_sessions =
-      $module->searchOn( $moduleOptions, "_nameID", $name_id->dump );
+      Lemonldap::NG::Common::Apache::Session->searchOn( $self->amOpts,
+        "_nameID", $name_id->dump );
 
-    if ( my @saml_sessions_keys = keys %$saml_sessions ) {
+    if (
+        my @saml_sessions_keys =
+        grep { $saml_sessions->{$_}->{_session_kind} eq $self->sessionKind }
+        keys %$saml_sessions
+      )
+    {
 
         # Warning if more than one session found
         if ( $#saml_sessions_keys > 0 ) {
@@ -1888,4 +1910,35 @@ sub sendImage {
     return $self->p->staticFile( $req, "common/$img", 'image/png' );
 }
 
+# Normalize url to be tolerant to SAML Path
+# Usefull if SAML Path is a regex
+# @return normalized url
+sub normalize_url {
+    my ( $self, $url, $samlPath, $metadataUrl ) = @_;
+
+    my $initialPath = "";
+    my $finalPath   = "";
+
+    # Get current (bad) path
+    if ( $url =~ m/($samlPath)/ ) {
+        $initialPath = $1;
+    }
+
+    # Get destination (good) path
+    if ( $metadataUrl =~ m/($samlPath)/ ) {
+        $finalPath = $1;
+    }
+
+    if (    $initialPath ne ""
+        and $finalPath ne ""
+        and $initialPath ne $finalPath )
+    {
+        $self->logger->debug(
+            "Normalizing url path form $initialPath to $finalPath");
+        $url =~ s/$initialPath/$finalPath/;
+    }
+
+    return $url;
+}
+
 1;
