@@ -10,8 +10,9 @@ use Lemonldap::NG::Portal::Main::Constants ':all';
 use Encode;
 use Unicode::String qw(utf8);
 use Scalar::Util 'weaken';
+use utf8;
 
-our $VERSION = '1.9.991_01';
+our $VERSION  = '2.0.0';
 our $ppLoaded = 0;
 
 BEGIN {
@@ -74,7 +75,6 @@ sub new {
     weaken $self->{portal};
 
     # Setting default LDAP password storage encoding to utf-8
-    $self->{conf}->{ldapPwdEnc} ||= 'utf-8';
     return $self;
 }
 
@@ -90,6 +90,9 @@ sub new {
 # @return Net::LDAP::Message
 sub bind {
     my ( $self, $dn, %args ) = @_;
+
+    $self->{portal}->logger->debug("Call bind for $dn") if $dn;
+
     my $mesg;
     unless ($dn) {
         $dn = $self->{conf}->{managerDn};
@@ -206,16 +209,28 @@ sub userBind {
             if ( $resp->grace_authentications_remaining ) {
 
                 # TODO
-                $req->info( '<h3>'
-                      . $resp->grace_authentications_remaining
-                      . ' <span trmsg="ppGrace">authentications remaining, change your password!</span></h3>'
+                $self->info(
+                    $self->loadTemplate(
+                        'ldapPpGrace',
+                        params => {
+                            number => $resp->grace_authentications_remaining
+                        }
+                    )
                 );
             }
 
             if ( $resp->time_before_expiration ) {
-                $req->info( '<h3 trspan="authRemaining,'
-                      . $self->convertSec( $resp->time_before_expiration )
-                      . '"></h3>' );
+                $req->info(
+                    $self->loadTemplate(
+                        'simpleInfo',
+                        params => {
+                            trspan => 'authRemaining,'
+                              . $self->convertSec(
+                                $resp->time_before_expiration
+                              )
+                        }
+                    )
+                );
             }
 
             return PE_OK;
@@ -227,8 +242,7 @@ sub userBind {
             return PE_OK;
         }
     }
-    $self->{portal}
-      ->userLogger->warn("Bad password for $self->{portal}->{user}");
+    $self->{portal}->userLogger->warn("Bad password for $req->{user}");
     return PE_BADCREDENTIALS;
 }
 
@@ -249,6 +263,9 @@ sub userModifyPassword {
     my $err;
     my $mesg;
 
+    utf8::downgrade($dn);
+    $self->{portal}->logger->debug("Call modify password for $dn");
+
     # Adjust configuration for AD
     if ($ad) {
         $ppolicyControl    = 0;
@@ -568,16 +585,18 @@ sub ldap {
     return 0;
 }
 
-## @method string searchGroups(string base, string key, string value, string attributes)
+## @method string searchGroups(string base, string key, string value, string attributes, hashref dupcheck)
 # Get groups from LDAP directory
 # @param base LDAP search base
 # @param key Attribute name in group containing searched value
 # @param value Searched value
 # @param attributes to get from found groups (array ref)
+# @param dupcheck to get from found groups (hash ref)
 # @return hashRef groups
 sub searchGroups {
-    my ( $self, $base, $key, $value, $attributes ) = @_;
+    my ( $self, $base, $key, $value, $attributes, $dupcheck ) = @_;
 
+    $dupcheck ||= {};
     my $groups = {};
 
     # Creating search filter
@@ -588,7 +607,7 @@ sub searchGroups {
     }
     $searchFilter .= "))";
 
-    $self->{p}->logger->debug("Group search filter: $searchFilter");
+    $self->{portal}->logger->debug("Group search filter: $searchFilter");
 
     # Search
     my $mesg = $self->search(
@@ -602,7 +621,7 @@ sub searchGroups {
 
         foreach my $entry ( $mesg->all_entries ) {
 
-            $self->{p}
+            $self->{portal}
               ->logger->debug( "Matching group " . $entry->dn() . " found" );
 
             # If recursive search is activated, do it here
@@ -616,16 +635,25 @@ sub searchGroups {
                 # Launch group search
                 if ($group_value) {
 
-                    $self->{p}
-                      ->logger->debug("Recursive search for $group_value");
+                    if ( $dupcheck->{$group_value} ) {
+                        $self->{portal}->logger->debug(
+"Disable search for $group_value, as it was already searched"
+                        );
+                    }
+                    else {
+                        $dupcheck->{$group_value} = 1;
+                        $self->{portal}
+                          ->logger->debug("Recursive search for $group_value");
 
-                    my $recursive_groups =
-                      $self->searchGroups( $base, $key, $group_value,
-                        $attributes );
+                        my $recursive_groups =
+                          $self->searchGroups( $base, $key, $group_value,
+                            $attributes, $dupcheck );
 
-                    my %allGroups = ( %$groups, %$recursive_groups )
-                      if ( ref $recursive_groups );
-                    $groups = \%allGroups;
+                        my %allGroups = ( %$groups, %$recursive_groups )
+                          if ( ref $recursive_groups );
+                        $groups = \%allGroups;
+
+                    }
                 }
             }
 
@@ -643,7 +671,7 @@ sub searchGroups {
                 my $data = $entry->get_value( $_, asref => 1 );
 
                 if ($data) {
-                    $self->{p}
+                    $self->{portal}
                       ->logger->debug("Store values of $_ in group $groupName");
                     $groups->{$groupName}->{$_} = $data;
                 }
