@@ -19,7 +19,7 @@ use Mouse;
 
 use Lemonldap::NG::Portal::Main::Constants qw(PE_OK PE_REDIRECT);
 
-our $VERSION = '1.9.991_01';
+our $VERSION = '2.0.0';
 
 # OpenID Connect standard claims
 use constant PROFILE => [
@@ -36,20 +36,28 @@ use constant PHONE => [qw/phone_number phone_number_verified/];
 has oidcOPList   => ( is => 'rw', default => sub { {} }, );
 has oidcRPList   => ( is => 'rw', default => sub { {} }, );
 has rpAttributes => ( is => 'rw', default => sub { {} }, );
+has spRules      => ( is => 'rw', default => sub { {} } );
 
 # return LWP::UserAgent object
 has ua => (
     is      => 'rw',
     lazy    => 1,
     builder => sub {
-
-        # TODO : LWP options to use a proxy for example
         my $ua = Lemonldap::NG::Common::UserAgent->new( $_[0]->{conf} );
         $ua->env_proxy();
         return $ua;
     }
 );
 
+has ott => (
+    is      => 'rw',
+    lazy    => 1,
+    default => sub {
+        my $ott = $_[0]->{p}->loadModule('::Lib::OneTimeToken');
+        return $ott;
+    }
+);
+
 # METHODS
 
 # Load OpenID Connect Providers and JWKS data
@@ -111,6 +119,17 @@ sub loadRPs {
             }
         }
         $self->rpAttributes->{$rp} = $attributes;
+
+        my $rule = $self->oidcRPList->{$rp}->{oidcRPMetaDataOptionsRule};
+        if ( length $rule ) {
+            $rule = $self->p->HANDLER->substitute($rule);
+            unless ( $rule = $self->p->HANDLER->buildSub($rule) ) {
+                $self->error( 'OIDC RP rule error: '
+                      . $self->p->HANDLER->tsv->{jail}->error );
+                return 0;
+            }
+            $self->spRules->{$rp} = $rule;
+        }
     }
     return 1;
 }
@@ -208,15 +227,6 @@ sub getCallbackUri {
       ? '&' . $callback_get_param . '=1'
       : '?' . $callback_get_param . '=1';
 
-    # Use authChoiceParam in redirect URL
-    if (    $self->conf->{authChoiceParam}
-        and $req->param( $self->conf->{authChoiceParam} ) )
-    {
-        $callback_uri .= '&'
-          . build_urlencoded( $self->{authChoiceParam} =>
-              $req->param( $self->conf->{authChoiceParam} ) );
-    }
-
     $self->logger->debug("OpenIDConnect Callback URI: $callback_uri");
     return $callback_uri;
 }
@@ -257,11 +267,7 @@ sub buildAuthorizationCodeAuthnRequest {
       ->{oidcOPMetaDataOptionsAcrValues};
 
     my $nonce;
-    if ($use_nonce) {
-        my $nonceSession =
-          $self->getOpenIDConnectSession( undef, { '_utime' => time } );
-        $nonce = $nonceSession->id;
-    }
+    $nonce = $self->ott->createToken if ($use_nonce);
 
     my $authn_uri =
         $authorize_uri
@@ -440,7 +446,7 @@ sub checkTokenResponseValidity {
     my ( $self, $json ) = @_;
 
     # token_type MUST be Bearer
-    unless ( $json->{token_type} eq "Bearer" ) {
+    unless ( $json->{token_type} =~ /^Bearer$/i ) {
         $self->logger->error(
             "Token type is " . $json->{token_type} . " but must be Bearer" );
         return 0;
@@ -530,15 +536,10 @@ sub checkIDTokenValidity {
         }
         else {
             # Get nonce session
-            my $nonceSession = $self->getOpenIDConnectSession($nonce);
-            unless ($nonceSession) {
+            unless ( $self->ott->getToken($nonce) ) {
                 $self->logger->error("Nonce $nonce verification failed");
                 return 0;
             }
-            else {
-                $nonceSession->remove;
-                $self->logger->debug("Nonce $nonce deleted");
-            }
         }
     }
 
@@ -618,7 +619,7 @@ sub decodeJSON {
     my ( $self, $json ) = @_;
     my $json_hash;
 
-    eval { $json_hash = from_json $json; };
+    eval { $json_hash = from_json( $json, { allow_nonref => 1 } ); };
 
     if ($@) {
         $json_hash->{error} = "parse_error";
@@ -632,15 +633,24 @@ sub decodeJSON {
 # @return Lemonldap::NG::Common::Session object
 sub getOpenIDConnectSession {
     my ( $self, $id, $info ) = @_;
+    my %storage = (
+        storageModule        => $self->conf->{oidcStorage},
+        storageModuleOptions => $self->conf->{oidcStorageOptions},
+    );
+    unless ( $storage{storageModule} ) {
+        %storage = (
+            storageModule        => $self->conf->{globalStorage},
+            storageModuleOptions => $self->conf->{globalStorageOptions},
+        );
+    }
 
     my $oidcSession = Lemonldap::NG::Common::Session->new(
         {
-            storageModule        => $self->conf->{oidcStorage},
-            storageModuleOptions => $self->conf->{oidcStorageOptions},
-            cacheModule          => $self->conf->{localSessionStorage},
-            cacheModuleOptions   => $self->conf->{localSessionStorageOptions},
-            id                   => $id,
-            kind                 => "OpenIDConnect",
+            %storage,
+            cacheModule        => $self->conf->{localSessionStorage},
+            cacheModuleOptions => $self->conf->{localSessionStorageOptions},
+            id                 => $id,
+            kind               => $self->sessionKind,
             ( $info ? ( info => $info ) : () ),
         }
     );
@@ -670,7 +680,7 @@ sub storeState {
     my $infos;
     foreach (@data) {
         $infos->{$_} = $req->{$_} if $req->{$_};
-        $infos->{"datas_$_"} = $req->datas->{$_} if $req->datas->{$_};
+        $infos->{"data_$_"} = $req->data->{$_} if $req->data->{$_};
     }
     return unless ($infos);
 
@@ -686,11 +696,7 @@ sub storeState {
     $infos->{_utime} = $time + ( $stateTimeout - $timeout );
 
     # Create state session and store infos
-    my $stateSession = $self->getOpenIDConnectSession( undef, $infos );
-    return unless $stateSession;
-
-    # Return session ID
-    return $stateSession->id;
+    return $self->ott->createToken($infos);
 }
 
 # Extract state information into $self
@@ -700,16 +706,16 @@ sub extractState {
     return 0 unless $state;
 
     # Open state session
-    my $stateSession = $self->getOpenIDConnectSession($state);
+    my $stateSession = $self->ott->getToken($state);
 
     return 0 unless $stateSession;
 
     # Push values in $self
-    foreach ( keys %{ $stateSession->data } ) {
+    foreach ( keys %{$stateSession} ) {
         next if $_ =~ /(type|_session_id|_session_kind|_utime)/;
-        my $tmp = $stateSession->data->{$_};
-        if (s/^datas_//) {
-            $req->datas->{$_} = $tmp;
+        my $tmp = $stateSession->{$_};
+        if (s/^data_//) {
+            $req->data->{$_} = $tmp;
         }
         elsif ( $req->can($_) ) {
             $req->$_($tmp);
@@ -719,15 +725,6 @@ sub extractState {
         }
     }
 
-    # Delete state session
-    if ( $stateSession->remove ) {
-        $self->logger->debug("State $state was deleted");
-    }
-    else {
-        $self->logger->error("Unable to delete state $state");
-        $self->logger->error( $stateSession->error );
-    }
-
     return 1;
 }
 
@@ -1062,7 +1059,7 @@ sub getEndPointAccessToken {
     my $access_token;
 
     my $authorization = $req->authorization;
-    if ( $authorization =~ /^Bearer (\w+)/i ) {
+    if ( $authorization and $authorization =~ /^Bearer (\w+)/i ) {
         $self->logger->debug("Bearer access token");
         $access_token = $1;
     }
@@ -1148,8 +1145,10 @@ sub createJWT {
 
     # JWT header
     my $jwt_header_hash = { typ => "JWT", alg => $alg };
-    $jwt_header_hash->{kid} = $self->conf->{oidcServiceKeyIdSig}
-      if $self->conf->{oidcServiceKeyIdSig};
+    if ( $alg eq "RS256" or $alg eq "RS384" or $alg eq "RS512" ) {
+        $jwt_header_hash->{kid} = $self->conf->{oidcServiceKeyIdSig}
+          if $self->conf->{oidcServiceKeyIdSig};
+    }
     my $jwt_header = encode_base64( to_json($jwt_header_hash), "" );
 
     if ( $alg eq "none" ) {
@@ -1263,7 +1262,8 @@ sub getIDTokenSub {
 sub getJWTJSONData {
     my ( $self, $jwt ) = @_;
     my $jwt_parts = $self->extractJWT($jwt);
-    return from_json( decode_base64url( $jwt_parts->[1] ) );
+    return from_json(
+        decode_base64url( $jwt_parts->[1], { allow_nonref => 1 } ) );
 }
 
 # Return JWKS representation of a key
@@ -1559,7 +1559,7 @@ L<Lemonldap::NG::Portal::AuthOpenIDConnect>, L<Lemonldap::NG::Portal::UserDBOpen
 =head1 BUG REPORT
 
 Use OW2 system to report bug or ask for features:
-L<http://jira.ow2.org>
+L<https://gitlab.ow2.org/lemonldap-ng/lemonldap-ng/issues>
 
 =head1 DOWNLOAD
 
