@@ -9,11 +9,15 @@ use strict;
 use Mouse;
 use JSON qw(from_json);
 use String::Random;
-use MIME::Lite;
+use MIME::Entity;
+use Email::Sender::Simple qw(sendmail);
+use Email::Sender::Transport::SMTP qw();
 use MIME::Base64;
 use Encode;
 
-our $VERSION = '1.9.991_01';
+our $VERSION = '2.0.0';
+
+our $transport;
 
 # PROPERTIES
 
@@ -25,8 +29,58 @@ has random => (
 );
 has charset => (
     is      => 'rw',
+    lazy    => 1,
     default => sub { return $_[0]->{conf}->{mailCharset} || 'utf-8' }
 );
+has transport => (
+    is      => 'rw',
+    lazy    => 1,
+    default => sub {
+        return $transport if $transport;
+        my $conf = $_[0]->{conf};
+        return undef
+          unless ( $conf->{SMTPServer} );
+        if (    $conf->{SMTPTLS}
+            and $Email::Sender::Simple::VERSION < 1.300027 )
+        {
+            require Email::Sender::Transport::SMTPS;
+            $transport = Email::Sender::Transport::SMTPS->new(
+                host => $conf->{SMTPServer},
+                ( $conf->{SMTPPort} ? ( port => $conf->{SMTPPort} ) : () ),
+                (
+                    $conf->{SMTPAuthUser}
+                    ? (
+                        sasl_username => $conf->{SMTPAuthUser},
+                        sasl_password => $conf->{SMTPAuthPass}
+                      )
+                    : ()
+                ),
+                ssl => $conf->{SMTPTLS},
+            );
+        }
+        else {
+            $transport = Email::Sender::Transport::SMTP->new(
+                host => $conf->{SMTPServer},
+                ( $conf->{SMTPPort} ? ( port => $conf->{SMTPPort} ) : () ),
+                (
+                    $conf->{SMTPAuthUser}
+                    ? (
+                        sasl_username => $conf->{SMTPAuthUser},
+                        sasl_password => $conf->{SMTPAuthPass}
+                      )
+                    : ()
+                ),
+                ( $conf->{SMTPTLS} ? ( ssl => $conf->{SMTPTLS} ) : () ),
+                (
+                    $conf->{SMTPTLSOpts}
+                    ? ( ssl_options => $conf->{SMTPTLSOpts} )
+                    : ()
+                ),
+            );
+        }
+        return $transport;
+    },
+);
 
 sub translate {
     my ( $self, $req ) = @_;
@@ -36,10 +90,13 @@ sub translate {
     my $json = $self->conf->{templateDir} . "/common/mail/$lang.json";
     $json = $self->conf->{templateDir} . '/common/mail/en.json'
       unless ( -f $json );
-    open F, $json or die $!;
+    open F, $json
+      or die 'Installation error: '
+      . $!
+      . " ($self->{conf}->{templateDir}/$lang.json or $self->{conf}->{templateDir}/common/mail/en.json)";
     $json = join '', <F>;
     close F;
-    $lang = from_json($json);
+    $lang = from_json( $json, { allow_nonref => 1 } );
     return sub {
         ($_) = @_;
         $$_ =~ s/\s+trspan="(\w+?)"(.*?)>.*?</"$2>".($lang->{$1}||$1).'<'/gse;
@@ -95,7 +152,7 @@ sub send_mail {
 
         # HTML case
         if ($html) {
-            $message = MIME::Lite->new(
+            $message = MIME::Entity->build(
                 From => $self->conf->{mailFrom},
                 To   => $mail,
                 (
@@ -117,7 +174,7 @@ sub send_mail {
             foreach ( keys %cid ) {
                 $message->attach(
                     Type => "image/" . ( $cid{$_} =~ m/\.(\w+)/ )[0],
-                    Id => $_,
+                    Id   => $_,
                     Path => $self->p->{templateDir} . "/" . $cid{$_},
                 );
             }
@@ -125,7 +182,7 @@ sub send_mail {
 
         # Plain text case
         else {
-            $message = MIME::Lite->new(
+            $message = MIME::Entity->build(
                 From       => $self->conf->{mailFrom},
                 To         => $mail,
                 "Reply-To" => $self->conf->{mailReplyTo},
@@ -141,13 +198,8 @@ sub send_mail {
         }
 
         # Send the mail
-        $self->conf->{SMTPServer}
-          ? $message->send(
-            "smtp", $self->conf->{SMTPServer},
-            AuthUser => $self->conf->{SMTPAuthUser},
-            AuthPass => $self->conf->{SMTPAuthPass}
-          )
-          : $message->send();
+        sendmail( $message->stringify,
+            ( $self->transport ? { transport => $self->transport } : () ) );
     };
     if ($@) {
         $self->logger->error("Send message failed: $@");
@@ -200,7 +252,9 @@ sub getRegisterSession {
     foreach my $id ( keys %$sessions ) {
         my $registerSession = $self->p->getApacheSession($id);
         next unless ($registerSession);
-        return $id if ( $registerSession->data->{_type} =~ /^register$/ );
+        return $id
+          if (  $registerSession->data->{_type}
+            and $registerSession->data->{_type} =~ /^register$/ );
     }
 
     # No register session found, return empty string
