@@ -3,13 +3,14 @@ package Lemonldap::NG::Portal::Main::Constants;
 use strict;
 use Exporter 'import';
 
-our $VERSION = '1.9.991_01';
+our $VERSION = '2.0.0';
 
 use constant HANDLER => 'Lemonldap::NG::Handler::PSGI::Main';
 use constant {
 
     # Portal errors
     # Developers warning, do not use PE_INFO, it's reserved to autoRedirect.
+    PE_IDPCHOICE                        => -5,
     PE_SENDRESPONSE                     => -4,
     PE_INFO                             => -3,
     PE_REDIRECT                         => -2,
@@ -88,6 +89,10 @@ use constant {
     PE_NOTOKEN                          => 81,
     PE_TOKENEXPIRED                     => 82,
     PE_U2FFAILED                        => 83,
+    PE_UNAUTHORIZEDPARTNER              => 84,
+    PE_RENEWSESSION                     => 85,
+    PE_WAIT                             => 86,
+    PE_MUSTAUTHN                        => 87,
 };
 
 # EXPORTER PARAMETERS
@@ -113,6 +118,7 @@ our @EXPORT_OK = qw( PE_SENDRESPONSE PE_INFO PE_REDIRECT PE_DONE PE_OK
   PE_RADIUSCONNECTFAILED PE_MUST_SUPPLY_OLD_PASSWORD PE_FORBIDDENIP
   PE_CAPTCHAERROR PE_CAPTCHAEMPTY PE_REGISTERFIRSTACCESS PE_REGISTERFORMEMPTY
   PE_REGISTERALREADYEXISTS PE_NOTOKEN PE_TOKENEXPIRED HANDLER PE_U2FFAILED
+  PE_UNAUTHORIZEDPARTNER PE_RENEWSESSION PE_IDPCHOICE PE_WAIT PE_MUSTAUTHN
 );
 our %EXPORT_TAGS = ( 'all' => [ @EXPORT_OK, 'import' ], );
 
