@@ -7,7 +7,7 @@ use utf8;
 use Mouse;
 use Clone 'clone';
 
-our $VERSION = '1.9.991_01';
+our $VERSION = '2.0.0';
 
 extends 'Lemonldap::NG::Common::Module';
 
@@ -15,22 +15,27 @@ extends 'Lemonldap::NG::Common::Module';
 
 has menuModules => (
     is      => 'rw',
+    lazy    => 1,
     builder => sub {
-        my $conf = $_[0]->{conf}->{menuModule};
+        my $conf = $_[0]->{conf};
         my @res;
-        foreach (qw(Appslist ChangePassword LoginHistory Logout)) {
+        foreach (qw(Appslist ChangePassword LoginHistory OidcConsents Logout)) {
             my $cond = $conf->{"portalDisplay$_"} // 1;
             $_[0]->p->logger->debug("Evaluate condition $cond for module $_");
             my $tmp =
-              $_[0]->{p}->HANDLER->buildSub($cond);
+              $_[0]->{p}
+              ->HANDLER->buildSub( $_[0]->{p}->HANDLER->substitute($cond) );
             push @res, [ $_, $tmp ] if ($tmp);
         }
         return \@res;
     }
 );
 
+has specific => ( is => 'rw', default => sub { {} } );
+
 has imgPath => (
     is      => 'rw',
+    lazy    => 1,
     builder => sub {
         return $_[0]->{conf}->{impgPath}
           || $_[0]->{conf}->{staticPrefix} . '/logos';
@@ -89,10 +94,16 @@ sub params {
             $req->param("tab") // '' )
           || "applist";
     }
+
     $res{DISPLAY_MODULES} = $self->displayModules($req);
     $res{AUTH_ERROR_TYPE} =
       $req->error_type( $res{AUTH_ERROR} = $req->menuError );
 
+    # Display menu 2fRegisters link only if at least a 2F device is registered
+    $res{sfaManager} =
+      $self->p->_sfEngine->display2fRegisters( $req, $req->userData );
+    $self->logger->debug("Display 2fRegisters link") if $res{sfaManager};
+
     return %res;
 }
 
@@ -108,7 +119,7 @@ sub displayModules {
     foreach my $module ( @{ $self->menuModules } ) {
         $self->logger->debug("Check if $module->[0] has to be displayed");
 
-        if ( $module->[1]->() ) {
+        if ( $module->[1]->( $req, $req->sessionInfo ) ) {
             my $moduleHash = { $module->[0] => 1 };
             if ( $module->[0] eq 'Appslist' ) {
                 $moduleHash->{'APPSLIST_LOOP'} = $self->appslist($req);
@@ -116,13 +127,17 @@ sub displayModules {
             elsif ( $module->[0] eq 'LoginHistory' ) {
                 $moduleHash->{'SUCCESS_LOGIN'} =
                   $self->p->mkSessionArray(
-                    $req->{sessionInfo}->{loginHistory}->{successLogin},
+                    $req->{sessionInfo}->{_loginHistory}->{successLogin},
                     "", 0, 0 );
                 $moduleHash->{'FAILED_LOGIN'} =
                   $self->p->mkSessionArray(
-                    $req->{sessionInfo}->{loginHistory}->{failedLogin},
+                    $req->{sessionInfo}->{_loginHistory}->{failedLogin},
                     "", 0, 1 );
             }
+            elsif ( $module->[0] eq 'OidcConsents' ) {
+                $moduleHash->{'OIDC_CONSENTS'} =
+                  $self->p->mkOidcConsent( $req->sessionInfo );
+            }
             push @$displayModules, $moduleHash;
         }
     }
@@ -250,147 +265,6 @@ sub _buildApplicationHash {
     return $applicationHash;
 }
 
-## @method string _displayConfCategory(string catname, hashref cathash, int catlevel)
-# Creates and returns HTML code for a category.
-# @param catname Category name
-# @param cathash Hash of category elements
-# @param catlevel Category level
-# @return HTML string
-sub _displayConfCategory {
-    my ( $self, $catname, $cathash, $catlevel ) = @_;
-    my $html;
-    my $key;
-
-    # Init HTML list
-    $html .= "<ul class=\"category cat-level-$catlevel\">\n";
-    $html .= "<li class=\"catname\">\n";
-    $html .= "<span>$catname</span>\n" if $catname;
-
-    # Increase category level
-    $catlevel++;
-
-    # Extract applications from hash
-    my $apphash;
-    foreach $key ( keys %$cathash ) {
-        next if $key =~ /(type|options|catname)/;
-        if (    $cathash->{$key}->{type}
-            and $cathash->{$key}->{type} eq "application" )
-        {
-            $apphash->{$key} = $cathash->{$key};
-        }
-    }
-
-    # display applications first
-    if ( scalar keys %$apphash > 0 ) {
-        $html .= "<ul>";
-        foreach $key ( keys %$apphash ) {
-            $html .= $self->_displayConfApplication( $key, $apphash->{$key} );
-        }
-        $html .= "</ul>";
-    }
-
-    # Display subcategories
-    foreach $key ( keys %$cathash ) {
-        next if $key =~ /(type|options|catname)/;
-        if (    $cathash->{$key}->{type}
-            and $cathash->{$key}->{type} eq "category" )
-        {
-            $html .=
-              $self->_displayConfCategory( $key, $cathash->{$key}, $catlevel );
-        }
-    }
-
-    # Close HTML list
-    $html .= "</li>\n";
-    $html .= "</ul>\n";
-
-    return $html;
-}
-
-## @method private string _displayConfApplication(string appid, hashref apphash)
-# Creates HTML code for an application.
-# @param $appid Application ID
-# @param $apphash Hash of application elements
-# @return HTML string
-sub _displayConfApplication {
-    my $self = shift;
-    my ( $appid, $apphash ) = @_;
-    my $html;
-    my $key;
-
-    # Get application items
-    my $appname = $apphash->{options}->{name} || $appid;
-    my $appuri  = $apphash->{options}->{uri}  || "";
-
-    # Display application
-    $html .=
-        "<li title=\"$appid\" class=\"appname $appid\"><span>"
-      . ( $appuri ? "<a href=\"$appuri\">$appname</a>" : "<a>$appname</a>" )
-      . "</span>\n";
-
-    # Detect sub applications
-    my $subapphash;
-    foreach $key ( keys %$apphash ) {
-        next if $key =~ /(type|options|catname)/;
-        if ( $apphash->{$key}->{type} eq "application" ) {
-            $subapphash->{$key} = $apphash->{$key};
-        }
-    }
-
-    # Display sub applications
-    if ( scalar keys %$subapphash > 0 ) {
-        $html .= "<ul>";
-        foreach $key ( keys %$subapphash ) {
-            $html .=
-              $self->_displayConfApplication( $key, $subapphash->{$key} );
-        }
-        $html .= "</ul>";
-    }
-
-    $html .= "</li>";
-    return $html;
-}
-
-## @method private string _displayConfDescription(string appid, hashref apphash)
-# Create HTML code for application description.
-# @param $appid Application ID
-# @param $apphash Hash
-# @return HTML string
-sub _displayConfDescription {
-    my $self = shift;
-    my ( $appid, $apphash ) = @_;
-    my $html = "";
-    my $key;
-
-    if ( defined $apphash->{type} and $apphash->{type} eq "application" ) {
-
-        # Get application items
-        my $appname = $apphash->{options}->{name} || $appid;
-        my $appuri  = $apphash->{options}->{uri}  || "";
-        my $appdesc = $apphash->{options}->{description};
-        my $applogofile = $apphash->{options}->{logo};
-        my $applogo     = $self->imgPath . $applogofile
-          if $applogofile;
-
-        # Display application description
-        $html .= "<div id=\"$appid\" class=\"appsdesc\">\n";
-        $html .=
-"<a href=\"$appuri\"><img src=\"$applogo\" alt=\"$appid logo\" /></a>\n"
-          if $applogofile;
-        $html .= "<p class=\"appname\">$appname</p>\n" if defined $appname;
-        $html .= "<p class=\"appdesc\">$appdesc</p>\n" if defined $appdesc;
-        $html .= "</div>\n";
-    }
-
-    # Sublevels
-    foreach $key ( keys %$apphash ) {
-        next if $key =~ /(type|options|catname)/;
-        $html .= $self->_displayConfDescription( $key, $apphash->{$key} );
-    }
-
-    return $html;
-}
-
 ## @method private string _filter(hashref apphash)
 # Duplicate hash reference
 # Remove unauthorized menu elements
@@ -450,6 +324,7 @@ sub _filterHash {
             my ( $vhost, $appuri ) =
               $apphash->{$key}->{options}->{uri} =~ m#^https?://([^/]*)(.*)#;
             $vhost =~ s/:\d+$//;
+            $vhost = $self->p->HANDLER->resolveAlias($vhost);
             $appuri ||= '/';
 
             # Remove if display is "no" or "off"
@@ -458,12 +333,40 @@ sub _filterHash {
             # Keep node if display is "yes" or "on"
             next if ( $appdisplay =~ /^(yes|on)$/ );
 
+            my $cond = undef;
+
+            # Handle partner rules (SAML, CAS or OIDC)
+            if ( $appdisplay =~ /^sp:\s*(.*)$/ ) {
+                my $p = $1;
+                if ( my $sub = $self->p->spRules->{$p} ) {
+                    eval {
+                        delete $apphash->{$key}
+                          unless ( $sub->( $req, $req->sessionInfo ) );
+                    };
+                    if ($@) {
+                        $self->logger->error("Partner rule $p returns: $@");
+                    }
+                }
+                next;
+            }
+
+            # If a specific rule exists, get it from cache or compile it
+            if ( $appdisplay !~ /^auto$/i ) {
+                if ( $self->specific->{$appuri} ) {
+                    $cond = $self->specific->{$appuri};
+                }
+                else {
+                    $cond = $self->specific->{$appuri} =
+                      $self->p->HANDLER->buildSub(
+                        $self->p->HANDLER->substitute($appdisplay) );
+                }
+            }
+
             # Check grant function if display is "auto" (this is the default)
             delete $apphash->{$key}
               unless (
                 $self->p->HANDLER->grant(
-                    $req->sessionInfo,
-                    $appuri, undef, $vhost
+                    $req, $req->sessionInfo, $appuri, $cond, $vhost
                 )
               );
             next;
