@@ -4,8 +4,9 @@ package Lemonldap::NG::Portal::Main::Plugin;
 
 use strict;
 use Mouse;
+use HTML::Template;
 
-our $VERSION = '1.9.991_01';
+our $VERSION = '2.0.0';
 
 extends 'Lemonldap::NG::Common::Module';
 
@@ -50,6 +51,11 @@ sub _addRoute {
     return $self;
 }
 
+sub loadTemplate {
+    my $self = shift;
+    return $self->p->loadTemplate(@_);
+}
+
 1;
 __END__
 
@@ -73,7 +79,7 @@ L<Lemonldap::NG::Portal> modules I<(plugins, authentication modules,...)>.
   sub init {
       my ($self) = @_;
       $self->addUnauthRoute( mypath => 'hello', [ 'GET', 'PUT' ] );
-      $self->addAuthRoute( mypath => 'wellcome', [ 'GET', 'PUT' ] );
+      $self->addAuthRoute( mypath => 'welcome', [ 'GET', 'PUT' ] );
       return 1;
   }
   sub verifyIP {
@@ -86,18 +92,18 @@ L<Lemonldap::NG::Portal> modules I<(plugins, authentication modules,...)>.
       ...
       return $self->p->sendJSONresponse($req, { hello => 1 });
   }
-  sub wellcome {
+  sub welcome {
       my ($self, $req) = @_;
       ...
-      return $self->p->sendHtml($req, 'template', params => { WELLCOME => 1 });
+      return $self->p->sendHtml($req, 'template', params => { WELCOME => 1 });
   }
 
 =head1 DESCRIPTION
 
-Lemonldap::NG::Portal::Main::Plugin provides many methods to write easily
+Lemonldap::NG::Portal::Main::Plugin provides many methods to easily write 
 Lemonldap::NG addons.
 
-init() is called for each plugin. If one plugin initialization fails (init()
+init() is called for each plugin. If a plugin initialization fails (init()
 returns 0), the portal responds a 500 status code for each request.
 
 =head1 Writing plugins
@@ -114,10 +120,10 @@ Plugins must be valid packages well found in C<@INC>.
 
 =head3 Entry point based on PATH_INFO
 
-Plugins can declare unauthRoutes/authRoutes during this initialization (=
+Plugins can declare unauthRoutes/authRoutes during initialization (=
 /path/info). Methods declared in this way must be declared in the plugin class.
-They will be called this the $req argument: the HTTP request
-(L<Lemonldap::NG::Portal::Main::Request>). These methods must return a valid
+They will be called with $req argument. $req is the HTTP request.
+(See L<Lemonldap::NG::Portal::Main::Request>). These methods must return a valid
 L<PSGI> response. You can also use sendJSONresponse() or sendHtml() methods
 (see L<Lemonldap::NG::Common::PSGI>).
 
@@ -126,7 +132,7 @@ Example:
   sub init {
       my ($self) = @_;
       $self->addUnauthRoute( mypath => 'hello', [ 'GET', 'PUT' ] );
-      $self->addAuthRoute( mypath => 'wellcome', [ 'GET', 'PUT' ] );
+      $self->addAuthRoute( mypath => 'welcome', [ 'GET', 'PUT' ] );
       return 1;
   }
   sub hello {
@@ -134,7 +140,7 @@ Example:
       ...
       return $self->p->sendJSONresponse($req, { hello => 1 });
   }
-  sub wellcome {
+  sub welcome {
       my ($self, $req) = @_;
       ...
       return $self->p->sendHtml($req, 'template', params => { WELLCOME => 1 });
@@ -143,19 +149,23 @@ Example:
 =head3 Entry point in auth process
 
 A plugin which wants to be inserted in authentication process has to declare
-constants containing the name of the method to run. The following entry points
-are available.
+constants set with method name to run. Following entry points are available.
 
 =over
 
 =item C<beforeAuth>: method called before authentication process
 
-=item C<betweenAuthAndDatas>: method called after authentication and before
+=item C<betweenAuthAndData>: method called after authentication and before
 setting C<sessionInfo> provisionning
 
-=item C<afterDatas>: method called after C<sessionInfo> provisionning
+=item C<afterData>: method called after C<sessionInfo> provisionning
 I<(macros, groups,...)>
 
+=item C<endAuth>: method called when session is validated (after cookie build)
+
+=item C<authCancel>: method called when user click on "cancel" during auth
+process
+
 =item C<forAuthUser>: method called for already authenticated users
 
 =item C<beforeLogout>: method called before logout
@@ -173,10 +183,16 @@ L<http://lemonldap-ng.org>
 
 =over
 
-=item Writing an authentication module: L<Lemonldap::NG::Portal::Auth::Base>
+=item Writing an authentication module: L<Lemonldap::NG::Portal::Auth>
+
+=item Writing a UserDB module: L<Lemonldap::NG::Portal::UserDB>
+
+=item Writing a second factor module: L<Lemonldap::NG::Portal::Main::SecondFactor>
 
 =item Writing an issuer module: L<Lemonldap::NG::Portal::Main::Issuer>
 
+=item Writing another plugin: L<Lemonldap::NG::Portal::Main::Plugin>
+
 =item Request object: L<Lemonldap::NG::Portal::Main::Request>
 
 =item Adding parameters in the manager: L<Lemonldap::NG::Manager::Build>
@@ -194,7 +210,7 @@ L<http://lemonldap-ng.org>
 =head1 BUG REPORT
 
 Use OW2 system to report bug or ask for features:
-L<http://jira.ow2.org>
+L<https://gitlab.ow2.org/lemonldap-ng/lemonldap-ng/issues>
 
 =head1 DOWNLOAD
 
