@@ -2,7 +2,7 @@
 # into "plugins" list in lemonldap-ng.ini, section "portal"
 package Lemonldap::NG::Portal::Main::Plugins;
 
-our $VERSION = '1.9.991_01';
+our $VERSION = '2.0.0';
 
 package Lemonldap::NG::Portal::Main;
 
@@ -11,18 +11,20 @@ use Mouse;
 
 # Plugins enabled by a simple boolean value (ordered list)
 #
-# Developers: U2F must be loaded before Notifications
+# Developers: 2FA must be loaded before Notifications
 our @pList = (
     portalDisplayResetPassword => '::Plugins::MailReset',
     portalStatus               => '::Plugins::Status',
     cda                        => '::Plugins::CDA',
-    portalForceAuthn           => '::Plugins::ForceAuth',
-    u2fActivation              => '::Plugins::U2F',
-    grantSessionRule           => '::Plugins::GrantSession',
-    u2fSelfRegistration        => '::Register::U2F',
     notification               => '::Plugins::Notifications',
     portalCheckLogins          => '::Plugins::History',
     stayConnected              => '::Plugins::StayConnected',
+    bruteForceProtection       => '::Plugins::BruteForceProtection',
+    grantSessionRules          => '::Plugins::GrantSession',
+    upgradeSession             => '::Plugins::Upgrade',
+    autoSigninRules            => '::Plugins::AutoSignin',
+    checkState                 => '::Plugins::CheckState',
+    portalForceAuthn           => '::Plugins::ForceAuthn',
 );
 
 ##@method list enabledPlugins
@@ -73,7 +75,7 @@ sub enabledPlugins {
     # TODO: change this name
     if ( $conf->{customPlugins} ) {
         $self->logger->debug( 'Custom plugins: ' . $conf->{customPlugins} );
-        push @res, grep ( /\w/, split( /,\s*/, $conf->{customPlugins} ) );
+        push @res, grep ( /\w+/, split( /,\s*/, $conf->{customPlugins} ) );
     }
     return @res;
 }
