@@ -7,7 +7,7 @@ use strict;
 use Mouse;
 use Lemonldap::NG::Portal::Main::Constants ':all';
 
-our $VERSION = '1.9.991_01';
+our $VERSION = '2.0.0';
 
 extends 'Lemonldap::NG::Common::PSGI::Request';
 
@@ -17,14 +17,14 @@ has steps => ( is => 'rw' );
 # Authentication result
 has authResult => ( is => 'rw' );
 
-# Datas shared between methods
-has datas => ( is => 'rw' );
-
-# Session datas when created
+# Session data when created
 has id          => ( is => 'rw' );
 has sessionInfo => ( is => 'rw' );
 has user        => ( is => 'rw' );
 
+# Persistent data (stored in cookie during auth, erased when auth is ready)
+has pdata => ( is => 'rw' );
+
 # Response cookies (list of strings built by cookie())
 has respCookies => ( is => 'rw' );
 
@@ -40,6 +40,10 @@ has customParameters => ( is => 'rw' );
 # Boolean to indicate that response must be a redirection
 has mustRedirect => ( is => 'rw' );
 
+# Boolean to indicate that login form must not be displayed (used to reset
+# authentication)
+has noLoginDisplay => ( is => 'rw' );
+
 # Store URL for redirections
 has urldc                  => ( is => 'rw' );
 has postUrl                => ( is => 'rw' );
@@ -112,7 +116,7 @@ sub error_type {
 
 sub init {
     my ($self) = @_;
-    $self->{$_} = {} foreach (qw(datas customParameters sessionInfo));
+    $self->{$_} = {} foreach (qw(data customParameters sessionInfo pdata));
     $self->{$_} = [] foreach (qw(respCookies));
 }
 
@@ -126,8 +130,8 @@ sub loginInfo {
 
 sub info {
     my ( $self, $info ) = @_;
-    $self->datas->{_info} .= $info if ( defined $info );
-    return $self->datas->{_info};
+    $self->data->{_info} .= $info if ( defined $info );
+    return $self->data->{_info};
 }
 
 sub addCookie {
@@ -135,6 +139,31 @@ sub addCookie {
     push @{ $self->respHeaders }, 'Set-Cookie' => $cookie;
 }
 
+sub delCookie {
+    my ( $self, $cookieName ) = @_;
+    my $i = 0;
+    @{ $self->respHeaders } = map {
+
+        # Look for a Set-Cookie header
+        if ( $_ =~ /^Set-Cookie$/i ) {
+            $i = 1;
+            return ();
+        }
+        elsif ($i) {
+
+            # Keep other cookies
+            unless (/^$cookieName\s*=/i) {
+                $i = 0;
+                return ( 'Set-Cookie' => $_ );
+            }
+
+            #Value is hidden here
+            return ();
+        }
+        return $_;
+    } @{ $self->respHeaders };
+}
+
 # TODO: oldpassword
 1;
 __END__
@@ -170,10 +199,17 @@ portal jobs.
 Stack of methods to call for this requests. It can be modified to change
 authentication process
 
-=head3 datas()
+=head3 data()
 
-Free hash ref where plugins can store their datas. Using it is a LLNG best
-practice
+Free hash ref where plugins can store their data (during one request). Using it
+is a LLNG best practice
+
+=head3 pdata
+
+Free hash ref where plugins can store some persistent data: data are kept
+during auth process and cleaned after successful authentication, except if
+C<$req-E<gt>pdata-E<gt>{keepPdata}> is set to 1. In this case, module that
+has set this flag must remove it after its job ends.
 
 =head3 User information
 
@@ -214,6 +250,19 @@ see notification plugin.
 
 Returns positive/warning/negative depending on value stored in error property.
 
+=head2 Cookie methods
+
+=over
+
+=item addCookie(string $cookie): add cookie in $req response headers. String
+is a complete cookie string, ex: "lemonldap=xxx"
+
+=item delCookie(string $cookieName): remove cookie from $req response headers.
+It doesn't remove navigator cookie but remove a "Set-Cookie" header if value
+match "L<lt>cookieNameL<gt>=..."
+
+=back
+
 =head1 SEE ALSO
 
 L<http://lemonldap-ng.org/>, L<Lemonldap::NG::Common::PSGI::Request>
@@ -229,7 +278,7 @@ L<http://lemonldap-ng.org/>, L<Lemonldap::NG::Common::PSGI::Request>
 =head1 BUG REPORT
 
 Use OW2 system to report bug or ask for features:
-L<http://jira.ow2.org>
+L<https://gitlab.ow2.org/lemonldap-ng/lemonldap-ng/issues>
 
 =head1 DOWNLOAD
 
