@@ -9,18 +9,23 @@
 #
 package Lemonldap::NG::Portal::Main::Run;
 
-our $VERSION = '1.9.991_01';
+our $VERSION = '2.0.0';
 
 package Lemonldap::NG::Portal::Main;
 
 use strict;
+use URI::Escape;
 
 # List constants
 sub authProcess { qw(extractFormInfo getUser authenticate) }
 
-sub sessionDatas {
+sub sessionData {
     qw(setAuthSessionInfo setSessionInfo setMacros setGroups setPersistentSessionInfo
-      setLocalGroups store buildCookie);
+      setLocalGroups store secondFactor);
+}
+
+sub validSession {
+    qw(storeHistory buildCookie);
 }
 
 # RESPONSE HANDLER
@@ -31,9 +36,38 @@ sub sessionDatas {
 # - launch Lemonldap::NG::Common::PSGI::Request::handler()
 sub handler {
     my ( $self, $req ) = @_;
+
     bless $req, 'Lemonldap::NG::Portal::Main::Request';
     $req->init();
-    return $self->Lemonldap::NG::Common::PSGI::Router::handler($req);
+    my $sp = 0;
+
+    # Restore pdata
+    if ( my $v = $req->cookies->{ $self->conf->{cookieName} . 'pdata' } ) {
+        $sp = 1;
+        eval { $req->pdata( JSON::from_json( uri_unescape($v) ) ); };
+        if ($@) {
+            $self->logger->error("Bad JSON content in cookie pdata");
+            $req->pdata( {} );
+        }
+    }
+    my $res = $self->Lemonldap::NG::Common::PSGI::Router::handler($req);
+
+    # Save pdata
+    if ( $sp or %{ $req->pdata } ) {
+        my %v = (
+            name => $self->conf->{cookieName} . 'pdata',
+            (
+                %{ $req->pdata }
+                ? ( value => uri_escape( JSON::to_json( $req->pdata ) ) )
+                : (
+                    value   => '',
+                    expires => 'Wed, 21 Oct 2015 00:00:00 GMT'
+                )
+            )
+        );
+        push @{ $res->[1] }, 'Set-Cookie', $self->cookie(%v);
+    }
+    return $res;
 }
 
 # MAIN ENTRY POINTS (declared in Lemonldap::NG::Portal::Main::Init)
@@ -62,8 +96,9 @@ sub login {
         $req,
         [
             'controlUrl',        @{ $self->beforeAuth },
-            $self->authProcess,  @{ $self->betweenAuthAndDatas },
-            $self->sessionDatas, @{ $self->afterDatas },
+            $self->authProcess,  @{ $self->betweenAuthAndData },
+            $self->sessionData,  @{ $self->afterData },
+            $self->validSession, @{ $self->endAuth },
         ]
     );
 }
@@ -73,10 +108,11 @@ sub postLogin {
     return $self->do(
         $req,
         [
-            'restoreArgs',                   'controlUrl',
-            @{ $self->beforeAuth },          $self->authProcess,
-            @{ $self->betweenAuthAndDatas }, $self->sessionDatas,
-            @{ $self->afterDatas },
+            'restoreArgs',                  'controlUrl',
+            @{ $self->beforeAuth },         $self->authProcess,
+            @{ $self->betweenAuthAndData }, $self->sessionData,
+            @{ $self->afterData },          $self->validSession,
+            @{ $self->endAuth },
         ]
     );
 }
@@ -86,8 +122,8 @@ sub authenticatedRequest {
     return $self->do(
         $req,
         [
-            'importHandlerDatas', 'controlUrl',
-            'checkLogout',        @{ $self->forAuthUser }
+            'importHandlerData', 'controlUrl',
+            'checkLogout',       @{ $self->forAuthUser }
         ]
     );
 }
@@ -97,8 +133,8 @@ sub postAuthenticatedRequest {
     return $self->do(
         $req,
         [
-            'importHandlerDatas', 'restoreArgs',
-            'controlUrl',         'checkLogout',
+            'importHandlerData', 'restoreArgs',
+            'controlUrl',        'checkLogout',
             @{ $self->forAuthUser }
         ]
     );
@@ -107,30 +143,40 @@ sub postAuthenticatedRequest {
 sub refresh {
     my ( $self, $req ) = @_;
     $req->mustRedirect(1);
-    my %datas = %{ $req->userData };
-    $req->user( $datas{ $self->conf->{whatToTrace} } );
-    $req->id( $datas{_session_id} );
+    my %data = %{ $req->userData };
+    $req->user( $data{ $self->conf->{whatToTrace} } );
+    $req->id( $data{_session_id} );
     $self->userLogger->notice( 'Refresh request for ' . $req->user );
-    foreach ( keys %datas ) {
-        delete $datas{$_} unless ( /^_/ or /^(?:startTime)$/ );
+    foreach ( keys %data ) {
+        delete $data{$_} unless ( /^_/ or /^(?:startTime)$/ );
     }
-    return $self->do(
-        $req,
+    $req->steps(
         [
             'getUser',
-            @{ $self->betweenAuthAndDatas },
+            @{ $self->betweenAuthAndData },
             'setAuthSessionInfo',
             'setSessionInfo',
             'setMacros',
             'setGroups',
             'setLocalGroups',
             sub {
-                $req->sessionInfo->{$_} = $datas{$_} foreach ( keys %datas );
+                $req->sessionInfo->{$_} = $data{$_} foreach ( keys %data );
                 return PE_OK;
             },
             'store',
         ]
     );
+    my $res = $req->error( $self->process($req) );
+    if ($res) {
+        $req->info(
+            $self->loadTemplate(
+                'simpleInfo', params => { trspan => 'rightsReloadNeedsLogout' }
+            )
+        );
+        $req->urldc( $self->conf->{portal} );
+        return $self->do( $req, [ sub { PE_INFO } ] );
+    }
+    return $self->do( $req, [ sub { PE_OK } ] );
 }
 
 sub logout {
@@ -150,19 +196,17 @@ sub logout {
 sub do {
     my ( $self, $req, $steps ) = @_;
     $req->steps($steps);
+    $req->data->{activeTimer} = $self->conf->{activeTimer};
     my $err = $req->error( $self->process($req) );
 
     # Update status
     if ( my $p = $self->HANDLER->tsv->{statusPipe} ) {
-        print $p ( $req->user ? $req->user : $req->address ) . ' => '
-          . $req->uri
-          . " $err\n";
+        $p->print( ( $req->user ? $req->user : $req->address ) . ' => '
+              . $req->uri
+              . " $err\n" );
     }
 
     # Update history
-    if ( $self->conf->{loginHistoryEnabled} ) {
-        $self->registerLogin($req);
-    }
     if ( $err == PE_SENDRESPONSE ) {
         return $req->response;
     }
@@ -171,14 +215,11 @@ sub do {
         if ( ( $err > 0 and !$req->id ) or $err eq PE_SESSIONNOTGRANTED ) {
             return [
                 401,
-                [
-                    'WWW-Authenticate' => "SSO " . $self->conf->{portal},
-                    'Access-Control-Allow-Origin' => '*'
-                ],
-                [qq'{"result":0,"error":$err"}']
+                [ 'WWW-Authenticate' => "SSO " . $self->conf->{portal} ],
+                [qq'{"result":0,"error":$err}']
             ];
         }
-        elsif ( $err > 0 ) {
+        elsif ( $err > 0 and $err != PE_PASSWORD_OK ) {
             return $self->sendJSONresponse(
                 $req,
                 { result => 0, error => $err },
@@ -202,9 +243,9 @@ sub do {
             and (
                 $err != PE_REDIRECT
                 or (    $err == PE_REDIRECT
-                    and $req->datas->{redirectFormMethod}
-                    and $req->datas->{redirectFormMethod} eq 'post' )
-                or ( $err == PE_REDIRECT and $req->info )
+                    and $req->data->{redirectFormMethod}
+                    and $req->data->{redirectFormMethod} eq 'post' )
+                or $req->info
             )
           )
         {
@@ -254,13 +295,15 @@ sub autoRedirect {
     my ( $self, $req ) = @_;
 
     # Set redirection URL if needed
-    $req->{urldc} ||= $self->conf->{portal} if ( $req->mustRedirect );
+    $req->{urldc} ||= $self->conf->{portal}
+      if ( $req->mustRedirect and not( $req->info ) );
 
     # Redirection should be made if urldc defined
-    if ( $req->{urldc} and not $req->param('lmError') ) {
-        if ( $self->_jsRedirect->() ) {
+    if ( $req->{urldc} ) {
+        $self->logger->debug("Building redirection to $req->{urldc}");
+        if ( $self->_jsRedirect->( $req, $req->sessionInfo ) ) {
             $req->error(PE_REDIRECT);
-            $req->datas->{redirectFormMethod} = "get";
+            $req->data->{redirectFormMethod} = "get";
         }
         else {
             return [ 302,
@@ -272,12 +315,11 @@ sub autoRedirect {
     return $self->sendHtml( $req, $tpl, params => $prms );
 }
 
-# Try to recover the session corresponding to id and return session datas.
+# Try to recover the session corresponding to id and return session data.
 # If $id is set to undef or if $args{force} is true, return a new session.
 sub getApacheSession {
     my ( $self, $id, %args ) = @_;
     $args{kind} ||= "SSO";
-
     if ($id) {
         $self->logger->debug("Try to get $args{kind} session $id");
     }
@@ -314,17 +356,22 @@ sub getApacheSession {
         $self->logger->debug("Session $args{kind} $id not found");
         return;
     }
-
+    $self->logger->debug("Get session $id from Portal::Main::Run") if ($id);
+    $self->logger->debug(
+        "Check session validity  -> " . $self->conf->{timeoutActivity} . "s" )
+      if ( $self->conf->{timeoutActivity} );
     my $now = time;
     if (
             $id
         and defined $as->data->{_utime}
         and (
-            $now - $as->data->{_utime} > $self->conf->{timeout}
-            or (    $self->conf->{timeoutActivity}
+            ( ( $now - $as->data->{_utime} ) > $self->conf->{timeout} )
+            or (
+                    $self->conf->{timeoutActivity}
                 and $as->data->{_lastSeen}
-                and $now - $as->data->{_lastSeen} >
-                $self->conf->{timeoutActivity} )
+                and ( ( $now - $as->data->{_lastSeen} ) >
+                    $self->conf->{timeoutActivity} )
+            )
         )
       )
     {
@@ -337,7 +384,7 @@ sub getApacheSession {
     return $as;
 }
 
-# Try to recover the persistent session corresponding to uid and return session datas.
+# Try to recover the persistent session corresponding to uid and return session data.
 sub getPersistentSession {
     my ( $self, $uid, $info ) = @_;
 
@@ -362,15 +409,17 @@ sub getPersistentSession {
     if ( $ps->error ) {
         $self->logger->debug( $ps->error );
     }
+    else {
 
-    # Set _session_uid if not already present
-    unless ( defined $ps->data->{_session_uid} ) {
-        $ps->update( { _session_uid => $uid } );
-    }
+        # Set _session_uid if not already present
+        unless ( defined $ps->data->{_session_uid} ) {
+            $ps->update( { _session_uid => $uid } );
+        }
 
-    # Set _utime if not already present
-    unless ( defined $ps->data->{_utime} ) {
-        $ps->update( { _utime => time } );
+        # Set _utime if not already present
+        unless ( defined $ps->data->{_utime} ) {
+            $ps->update( { _utime => time } );
+        }
     }
 
     return $ps;
@@ -388,9 +437,9 @@ sub updatePersistentSession {
 
     # Return if no infos to update
     return () unless ( ref $infos eq 'HASH' and %$infos );
-
     $uid ||= $req->{sessionInfo}->{ $self->conf->{whatToTrace} }
       || $req->userData->{ $self->conf->{whatToTrace} };
+    $self->logger->debug("Found 'whatToTrace' -> $uid");
     unless ($uid) {
         $self->logger->debug('No uid found, skipping updatePersistentSession');
         return ();
@@ -411,7 +460,7 @@ sub updatePersistentSession {
 
 # Update session stored.
 # If no id is given, try to get it from cookie.
-# If the session is available, update datas with $info.
+# If the session is available, update data with $info.
 # Note that outdated session data may remain some time on
 # server local cache, if there are several LL::NG servers.
 # @param infos hash reference of information to update
@@ -424,22 +473,22 @@ sub updateSession {
     return () unless ( ref $infos eq 'HASH' and %$infos );
 
     # Recover session ID unless given
-    $id ||= $req->{id};
+    $id ||= $req->{id} || $req->userData->{_session_id};
 
     if ($id) {
 
         # Update sessionInfo data
         ## sessionInfo updated if $id defined : quite strange !!
-        ## See http://jira.ow2.org/browse/LEMONLDAP-430
+        ## See https://gitlab.ow2.org/lemonldap-ng/lemonldap-ng/issues/430
         foreach ( keys %$infos ) {
             $self->logger->debug(
                 "Update sessionInfo $_ with " . $infos->{$_} );
-            $req->{sessionInfo}->{$_} = $self->HANDLER->datas->{$_} =
+            $req->{sessionInfo}->{$_} = $self->HANDLER->data->{$_} =
               $infos->{$_};
         }
 
-        # Update session in global storage with updateTime
-        $infos->{updateTime} = strftime( "%Y%m%d%H%M%S", localtime() );
+        # Update session in global storage with _updateTime
+        $infos->{_updateTime} = strftime( "%Y%m%d%H%M%S", localtime() );
         if ( my $apacheSession =
             $self->getApacheSession( $id, info => $infos ) )
         {
@@ -483,12 +532,12 @@ sub _deleteSession {
                 value   => 0,
                 domain  => $self->conf->{domain},
                 secure  => 0,
-                expires => '-1d',
+                expires => 'Wed, 21 Oct 2015 00:00:00 GMT'
             )
         ) unless ($preserveCookie);
     }
 
-    HANDLER->localUnlog( $session->id );
+    HANDLER->localUnlog( $req, $session->id );
     $session->remove;
 
     # Create an obsolete cookie to remove it
@@ -498,7 +547,7 @@ sub _deleteSession {
             value   => 0,
             domain  => $self->conf->{domain},
             secure  => 0,
-            expires => '-1d',
+            expires => 'Wed, 21 Oct 2015 00:00:00 GMT'
         )
     ) unless ($preserveCookie);
 
@@ -525,7 +574,9 @@ sub isTrustedUrl {
 sub stamp {
     my $self = shift;
     my $res =
-      $self->conf->{cipher} ? $self->conf->{cipher}->encrypt( time() ) : 1;
+        $self->conf->{cipher}
+      ? $self->conf->{cipher}->encrypt( time() )
+      : 1;
     $res =~ s/\+/%2B/g;
     return $res;
 }
@@ -550,7 +601,7 @@ sub autoPost {
         return PE_INFO;
     }
 
-    $req->datas->{redirectFormMethod} = "post";
+    $req->data->{redirectFormMethod} = "post";
     return PE_REDIRECT;
 }
 
@@ -567,11 +618,13 @@ sub setHiddenFormValue {
     # Default values
     $prefix = "lmhidden_" unless defined $prefix;
     $base64 = 1           unless defined $base64;
+    $val    = ''          unless defined $val;
 
     # Store value
-    if ($val) {
+    if ( defined $val or !( $val & ~$val ) ) {
         $key = $prefix . $key;
-        $val =~ s/\+/%2B/g;
+
+        #$val =~ s/\+/%2B/g;
         $req->{portalHiddenFormValues}->{$key} = $val;
         $self->logger->debug("Store $val in hidden key $key");
     }
@@ -593,10 +646,11 @@ sub getHiddenFormValue {
     $key = $prefix . $key;
 
     # Get value
-    if ( my $val = $req->param($key) ) {
+    my $val = $req->param($key);
+    if ( defined $val ) {
         $val = decode_base64($val) if $base64;
-        return $val;
         $self->logger->debug("Hidden value $val found for key $key");
+        return $val;
     }
 
     # No value found
@@ -643,7 +697,7 @@ sub fullUrl {
     my ( $self, $req ) = @_;
     my $pHost = $self->conf->{portal};
     $pHost =~ s#^(https?://[^/]+)(?:/.*)?$#$1#;
-    return $pHost . $req->uri;
+    return $pHost . $req->env->{REQUEST_URI};
 }
 
 sub cookie {
@@ -653,7 +707,8 @@ sub cookie {
     $res[0] .= "=$h{value}";
     $h{path} ||= '/';
     $h{HttpOnly} //= $self->conf->{httpOnly};
-    $h{expires}  //= $self->conf->{cookieExpiration};
+    $h{max_age}  //= $self->conf->{cookieExpiration}
+      if ( $self->conf->{cookieExpiration} );
     foreach (qw(domain path expires max_age HttpOnly)) {
         my $f = $_;
         $f =~ s/_/-/g;
@@ -666,32 +721,56 @@ sub cookie {
 sub _dump {
     my ( $self, $variable ) = @_;
     require Data::Dumper;
-    $Data::Dumper::Indent = 0;
+    $Data::Dumper::Indent  = 0;
+    $Data::Dumper::Useperl = 1;
     $self->logger->debug( "Dump: " . Data::Dumper::Dumper($variable) );
     return;
 }
 
 sub sendHtml {
     my ( $self, $req, $template, %args ) = @_;
-    push @{ $req->respHeaders },
+    my $res = $self->SUPER::sendHtml( $req, $template, %args );
+    push @{ $res->[1] },
       'X-XSS-Protection'       => '1; mode=block',
       'X-Content-Type-Options' => 'nosniff';
 
-    # Set authorizated URL for POST
-    my $csp = $self->csp . "form-action 'self'";
+    # Set authorized URL for POST
+    my $csp = $self->csp . "form-action " . $self->conf->{cspFormAction};
     if ( my $url = $req->urldc ) {
-        $url =~ s#https?://([^/]+).*#$1#;
+        $self->logger->debug("Required urldc : $url");
+        $url =~ s#(https?://[^/]+).*#$1#;
+        $self->logger->debug("Set CSP form-action with urldc : $url");
         $csp .= " $url";
     }
     my $url = $args{params}->{URL};
-    if ( $url and $url =~ s#https?://([^/]+).*#$1# ) {
-        $csp .= " $url";
+    if ( defined $url ) {
+        $self->logger->debug("Required Params URL : $url");
+        if ( $url =~ s#(https?://[^/]+).*#$1# ) {
+            $self->logger->debug("Set CSP form-action with Params URL : $url");
+            $csp .= " $url";
+        }
+    }
+    if ( defined $req->{cspFormAction} ) {
+        $self->logger->debug(
+            "Set CSP form-action with request URL: " . $req->{cspFormAction} );
+        $csp .= " " . $req->{cspFormAction};
+    }
+
+    # Set SAML Discovery Protocol in form-action
+    # See https://github.com/w3c/webappsec-csp/issues/8
+    if ( $self->conf->{samlDiscoveryProtocolActivation}
+        and defined $self->conf->{samlDiscoveryProtocolURL} )
+    {
+        $self->logger->debug(
+            "Add SAML Discovery Protocol URL in CSP form-action");
+
+        $csp .= " " . $self->conf->{samlDiscoveryProtocolURL};
     }
     $csp .= ';';
 
     # Deny using portal in frame except if it is required
     unless ( $req->frame or $self->conf->{portalAntiFrame} == 0 ) {
-        push @{ $req->respHeaders }, 'X-Frame-Options' => 'DENY';
+        push @{ $res->[1] }, 'X-Frame-Options' => 'DENY';
         $csp .= "frame-ancestors 'none';";
     }
 
@@ -706,9 +785,47 @@ sub sendHtml {
     }
 
     # Set CSP header
-    push @{ $req->respHeaders }, 'Content-Security-Policy' => $csp;
+    push @{ $res->[1] }, 'Content-Security-Policy' => $csp;
+    $self->logger->debug("Apply following CSP : $csp");
+    return $res;
+}
 
-    return $self->SUPER::sendHtml( $req, $template, %args );
+sub sendCss {
+    my ( $self, $req ) = @_;
+    my $s =
+        'html,body{background:url("'
+      . $self->staticPrefix
+      . '/common/backgrounds/'
+      . $self->conf->{portalSkinBackground}
+      . '") no-repeat center fixed;'
+      . 'background-size:cover;}';
+    return [
+        200,
+        [
+            'Content-Type'   => 'text/css',
+            'Content-Length' => length($s),
+            'Cache-Control'  => 'public,max-age=3600',
+        ],
+        [$s]
+    ];
+}
+
+sub lmError {
+    my ( $self, $req ) = @_;
+    my $httpError = $req->param('code');
+
+    # Check URL
+    $self->controlUrl($req);
+
+    my %templateParams = (
+        LOGOUT_URL => $self->conf->{portal} . "?logout=1",
+        URL        => $req->{urldc},
+    );
+
+    # Error code
+    $templateParams{"ERROR$_"} = ( $httpError == $_ ? 1 : 0 )
+      foreach ( 403, 404, 500, 502, 503 );
+    return $self->sendHtml( $req, 'error', params => \%templateParams );
 }
 
 sub rebuildCookies {
@@ -723,9 +840,16 @@ sub rebuildCookies {
 }
 
 sub tplParams {
+    my $portalPath = $_[0]->conf->{portal};
+    $portalPath =~ s#^https?://[^/]+/?#/#;
+    $portalPath =~ s#[^/]+\.fcgi$##;
     return (
         SKIN       => $_[0]->getSkin( $_[1] ),
         PORTAL_URL => $_[0]->conf->{portal},
+        SKIN_PATH  => $portalPath . "skins",
+        ANTIFRAME  => $_[0]->conf->{portalAntiFrame},
+        SKIN_BG    => $_[0]->conf->{portalSkinBackground},
+        ( $_[0]->customParameters ? ( %{ $_[0]->customParameters } ) : () ),
     );
 }
 
@@ -734,7 +858,7 @@ sub registerLogin {
     return
       unless ( $self->conf->{loginHistoryEnabled}
         and defined $req->authResult );
-    my $history = $req->sessionInfo->{loginHistory} ||= {};
+    my $history = $req->sessionInfo->{_loginHistory} ||= {};
     my $type = ( $req->authResult > 0 ? 'failed' : 'success' ) . 'Login';
     $history->{$type} ||= [];
     $self->logger->debug("Current login saved into $type");
@@ -744,6 +868,9 @@ sub registerLogin {
     $login->{error} = $self->error( $req->authResult )
       if ( $req->authResult );
 
+    $self->logger->debug( " Current login -> " . $login->{error} )
+      if ( $login->{error} );
+
     # Add current login into history
     unshift @{ $history->{$type} }, $login;
 
@@ -752,7 +879,9 @@ sub registerLogin {
       if ( scalar @{ $history->{$type} } > $self->conf->{ $type . "Number" } );
 
     # Save into persistent session
-    $self->updatePersistentSession( $req, { loginHistory => $history, } );
+    $self->updatePersistentSession( $req, { _loginHistory => $history, } );
+
+    PE_OK;
 }
 
 # put main session data into a hash ref
@@ -770,4 +899,31 @@ sub _sumUpSession {
     return $res;
 }
 
+# Temlate loader
+sub loadTemplate {
+    my ( $self, $name, %prm ) = @_;
+    $name .= '.tpl';
+    my $file =
+        $self->conf->{templateDir} . '/'
+      . $self->conf->{portalSkin} . '/'
+      . $name;
+    $file = $self->conf->{templateDir} . '/common/' . $name
+      unless ( -e $file );
+    unless ( -e $file ) {
+        die "Unable to find $name in $self->conf->{templateDir}";
+    }
+    my $tpl = HTML::Template->new(
+        filename               => $file,
+        die_on_bad_params      => 0,
+        die_on_missing_include => 1,
+        cache                  => 1,
+        global_vars            => 0,
+        ( $prm{filter} ? ( filter => $prm{filter} ) : () ),
+    );
+    if ( $prm{params} ) {
+        $tpl->param( %{ $prm{params} } );
+    }
+    return $tpl->output;
+}
+
 1;
