@@ -4,37 +4,42 @@ use strict;
 use Mouse;
 use Lemonldap::NG::Portal::Main::Constants qw(PE_INFO PE_OK);
 
-our $VERSION = '1.9.991_01';
+our $VERSION = '2.0.0';
 
 extends 'Lemonldap::NG::Portal::Main::Plugin',
   'Lemonldap::NG::Portal::Lib::OtherSessions';
 
-sub afterDatas { 'run' }
+# INITIALIZATION
+
+use constant endAuth => 'run';
 
 sub init { 1 }
 
+# RUNNING METHOD
+
 sub run {
     my ( $self, $req ) = @_;
     if ( $req->param('checkLogins') ) {
         $self->logger->debug('History asked');
         $req->info(
             (
-                $req->sessionInfo->{loginHistory}->{successLogin}
-                ? $self->mkSessionArray(
-                    $req->sessionInfo->{loginHistory}->{successLogin},
+                $req->sessionInfo->{_loginHistory}->{successLogin}
+                ? $self->p->mkSessionArray(
+                    $req->sessionInfo->{_loginHistory}->{successLogin},
                     'lastLogins', 0, 0 )
                 : ""
             )
-            . (
-                $req->sessionInfo->{loginHistory}->{failedLogin}
-                ? $self->mkSessionArray(
-                    $req->sessionInfo->{loginHistory}->{failedLogin},
+            . ("<hr>")
+              . (
+                $req->sessionInfo->{_loginHistory}->{failedLogin}
+                ? $self->p->mkSessionArray(
+                    $req->sessionInfo->{_loginHistory}->{failedLogin},
                     'lastFailedLogins', 0, 1 )
                 : ""
-            )
+              )
         );
         unless ( $req->info ) {
-            $req->info('<p trspan="noHistory"></p>');
+            $req->info( $self->loadTemplate('noHistory') );
         }
         return PE_INFO;
     }
