@@ -2,9 +2,9 @@ package Lemonldap::NG::Portal::Plugins::MailReset;
 
 use strict;
 use Encode;
-use HTML::Template;
 use Mouse;
 use POSIX qw(strftime);
+use Lemonldap::NG::Common::FormEncode;
 use Lemonldap::NG::Portal::Main::Constants qw(
   PE_BADCREDENTIALS
   PE_BADMAILTOKEN
@@ -22,12 +22,13 @@ use Lemonldap::NG::Portal::Main::Constants qw(
   PE_OK
   PE_PASSWORDFIRSTACCESS
   PE_PASSWORDFORMEMPTY
+  PE_PASSWORD_MISMATCH
   PE_PASSWORD_OK
   PE_TOKENEXPIRED
   PE_USERNOTFOUND
 );
 
-our $VERSION = '1.9.991_01';
+our $VERSION = '2.0.0';
 
 extends 'Lemonldap::NG::Portal::Main::Plugin',
   'Lemonldap::NG::Portal::Lib::SMTP';
@@ -38,6 +39,7 @@ extends 'Lemonldap::NG::Portal::Main::Plugin',
 # Form timout token generator (used even if requireToken is not set)
 has ott => (
     is      => 'rw',
+    lazy    => 1,
     default => sub {
         my $ott =
           $_[0]->{p}->loadModule('Lemonldap::NG::Portal::Lib::OneTimeToken');
@@ -70,6 +72,8 @@ sub init {
 sub resetPwd {
     my ( $self, $req ) = @_;
 
+    $self->p->controlUrl($req);
+
     # Check parameters
     $req->error( $self->_reset($req) );
 
@@ -80,23 +84,29 @@ sub resetPwd {
 
 sub _reset {
     my ( $self, $req ) = @_;
-    my ( $mailToken, $newPwd, $confirmPwd );
+    my ( $mailToken, $newPwd, $confirmPwd, %tplPrms );
 
     # PASSWORD CHANGE FORM => changePwd()
-    if ( $req->method =~ /^POST$/i
-        and ( $req->param('newpassword') or $req->param('reset') ) )
+    if (
+        $req->method =~ /^POST$/i
+        and (  $req->param('newpassword')
+            or $req->param('confirmpassword')
+            or $req->param('reset') )
+      )
     {
         return $self->changePwd($req);
     }
 
     # FIRST FORM
-    $mailToken = $req->datas->{mailToken} = $req->param('mail_token');
+    $mailToken = $req->data->{mailToken} = $req->param('mail_token');
     unless ( $req->param('mail') || $mailToken ) {
         $self->setSecurity($req);
         return PE_MAILFIRSTACCESS if ( $req->method eq 'GET' );
         return PE_MAILFORMEMPTY;
     }
 
+    my $searchByMail = 1;
+
     # OTHER FORMS
     if ($mailToken) {
         $self->logger->debug( "Token given for password reset: " . $mailToken );
@@ -108,18 +118,22 @@ sub _reset {
             return PE_BADMAILTOKEN;
         }
 
-        $req->{mail} = $mailSession->data->{user};
-        $req->datas->{mailAddress} =
+        $req->{user} = $mailSession->data->{user};
+        $req->data->{mailAddress} =
           $mailSession->data->{ $self->conf->{mailSessionKey} };
-        $self->logger->debug( 'User associated to: ' . $req->{mail} );
+        $self->logger->debug( 'User associated to: ' . $req->{user} );
+
+        # Restore pdata if any
+        $req->pdata( $mailSession->data->{_pdata} || {} );
         $mailSession->remove;
+        $searchByMail = 0 unless ( $req->{user} =~ /\@/ );
     }
 
     # Check for values posted
     else {
 
         # Use submitted value
-        $req->{mail} = $req->param('mail');
+        $req->{user} = $req->param('mail');
 
         # Check if token exists
         my $token;
@@ -157,18 +171,17 @@ sub _reset {
         elsif ( $self->conf->{requireToken} ) {
             unless ( $self->ott->getToken($token) ) {
                 $self->setSecurity($req);
-                $self->userLogger->warn('Reset try with expired/bas token');
+                $self->userLogger->warn('Reset try with expired/bad token');
                 return PE_TOKENEXPIRED;
             }
         }
-        unless ( $req->{mail} =~ /$self->{conf}->{userControl}/o ) {
+        unless ( $req->{user} =~ /$self->{conf}->{userControl}/o ) {
             $self->setSecurity($req);
             return PE_MALFORMEDUSER;
         }
     }
 
     # Search user in database
-    #$req->user( $req->{mail} );
     $req->steps(
         [
             'getUser',                  'setSessionInfo',
@@ -176,19 +189,33 @@ sub _reset {
             'setPersistentSessionInfo', 'setLocalGroups'
         ]
     );
-    if ( my $error = $self->p->process($req) ) {
-        if ( $error == PE_USERNOTFOUND or $error = PE_BADCREDENTIALS ) {
-            $self->userLogger->warn(
-                "Reset asked for a unvalid user ($req->{mail})");
+    if ( my $error = $self->p->process( $req, useMail => $searchByMail ) ) {
+        if ( $error == PE_USERNOTFOUND or $error == PE_BADCREDENTIALS ) {
+            $self->userLogger->warn( "Reset asked for a unvalid user ("
+                  . $req->param('mail')
+                  . ")" );
 
             # To avoid mail enumeration, return OK
+            # unless portalErrorOnMailNotFound is set
+
+            if ( $self->conf->{portalErrorOnMailNotFound} ) {
+                return PE_MAILNOTFOUND;
+            }
+
+            my $mailTimeout =
+              $self->conf->{mailTimeout} || $self->conf->{timeout};
+            my $expTimestamp = time() + $mailTimeout;
+            $req->data->{expMailDate} =
+              strftime( "%d/%m/%Y", localtime $expTimestamp );
+            $req->data->{expMailTime} =
+              strftime( "%H:%M", localtime $expTimestamp );
             return PE_MAILCONFIRMOK;
         }
         return $error;
     }
 
     # Build temporary session
-    my $mailSession = $self->getMailSession( $req->{mail} );
+    my $mailSession = $self->getMailSession( $req->{user} );
     unless ( $mailSession or $mailToken ) {
 
         # Create a new session
@@ -215,11 +242,14 @@ sub _reset {
             $req->{sessionInfo}->{ $self->conf->{mailSessionKey} } );
 
         # Store user
-        $infos->{user} = $req->{mail};
+        $infos->{user} = $req->{user};
 
         # Store type
         $infos->{_type} = "mail";
 
+        # Store pdata
+        $infos->{_pdata} = $req->pdata;
+
         # create session
         $mailSession = $self->p->getApacheSession( undef, info => $infos );
 
@@ -227,7 +257,8 @@ sub _reset {
     }
     elsif ($mailSession) {
         $self->logger->debug( 'Mail session found: ' . $mailSession->id );
-        $req->datas->{mailAlreadySent} = 1;
+        $req->id( $mailSession->id );
+        $req->data->{mailAlreadySent} = 1;
     }
 
     # Send confirmation mail
@@ -238,50 +269,54 @@ sub _reset {
 
         $self->logger->debug("Mail expiration timestamp: $expTimestamp");
 
-        $req->datas->{expMailDate} =
+        $req->data->{expMailDate} =
           strftime( "%d/%m/%Y", localtime $expTimestamp );
-        $req->datas->{expMailTime} =
+        $req->data->{expMailTime} =
           strftime( "%H:%M", localtime $expTimestamp );
 
         # Mail session start date
         my $startTimestamp = $mailSession->data->{mailSessionStartTimestamp};
 
         $self->logger->debug("Mail start timestamp: $startTimestamp");
-        $req->datas->{startMailDate} =
+        $req->data->{startMailDate} =
           strftime( "%d/%m/%Y", localtime $startTimestamp );
-        $req->datas->{startMailTime} =
+        $req->data->{startMailTime} =
           strftime( "%H:%M", localtime $startTimestamp );
 
         # Ask if user want another confirmation email
-        if ( $req->datas->{mailAlreadySent}
+        if ( $req->data->{mailAlreadySent}
             and !$req->param('resendconfirmation') )
         {
             $self->userLogger->notice(
-                'Reset mail already sent to ' . $req->{mail} );
+                'Reset mail already sent to ' . $req->{user} );
 
-            # To avoid enumeration, return OK
-            return PE_MAILCONFIRMOK;
+            # Return mail already sent only if it is allowed at previous step
+            if ( $self->conf->{portalErrorOnMailNotFound} ) {
+                $self->captcha(0);
+                $self->setSecurity($req);
+                return PE_MAILCONFIRMATION_ALREADY_SENT;
+            }
         }
 
         # Get mail address
-        $req->datas->{mailAddress} ||=
+        $req->data->{mailAddress} ||=
           $self->p->getFirstValue(
             $req->{sessionInfo}->{ $self->conf->{mailSessionKey} } );
-        return PE_MAILERROR unless ( $req->datas->{mailAddress} );
+        return PE_MAILERROR unless ( $req->data->{mailAddress} );
 
         # Build confirmation url
+        my $req_url = $req->data->{_url};
+        my $skin    = $self->p->getSkin($req);
         my $url =
-            $self->conf->{mailUrl}
-          . "?mail_token="
-          . $req->{id}
-          . '&skin='
-          . $self->p->getSkin($req);
-        $url .= '&'
-          . $self->conf->{authChoiceParam} . '='
-          . $req->datas->{_authChoice}
-          if ( $req->datas->{_authChoice} );
+          $self->conf->{mailUrl} . '?'
+          . build_urlencoded(
+            mail_token => $req->{id},
+            skin       => $skin,
+            ( $req_url ? ( url => $req_url ) : () ),
+          );
 
         # Build mail content
+        $tplPrms{MAIN_LOGO} = $self->conf->{portalMainLogo};
         my $tr      = $self->translate($req);
         my $subject = $self->conf->{mailConfirmSubject};
         unless ($subject) {
@@ -298,30 +333,24 @@ sub _reset {
         else {
 
             # Use HTML template
-            my $tplfile =
-                $self->conf->{templateDir} . '/'
-              . $self->conf->{portalSkin}
-              . '/mail_confirm.tpl';
-            $tplfile = $self->conf->{templateDir} . '/common/mail_confirm.tpl'
-              unless ( -e $tplfile );
-            my $template = HTML::Template->new(
-                filename => $tplfile,
-                filter   => $tr,
+            $body = $self->loadTemplate(
+                'mail_confirm',
+                filter => $tr,
+                params => \%tplPrms
             );
-            $body = $template->output();
             $html = 1;
         }
 
         # Replace variables in body
-        $body =~ s/\$expMailDate/$req->datas->{expMailDate}/ge;
-        $body =~ s/\$expMailTime/$req->datas->{expMailTime}/ge;
+        $body =~ s/\$expMailDate/$req->data->{expMailDate}/ge;
+        $body =~ s/\$expMailTime/$req->data->{expMailTime}/ge;
         $body =~ s/\$url/$url/g;
-        $body =~ s/\$(\w+)/$req->{sessionInfo}->{$1}/ge;
+        $body =~ s/\$(\w+)/$req->{sessionInfo}->{$1} || ''/ge;
 
         # Send mail
         unless (
             $self->send_mail(
-                $req->datas->{mailAddress},
+                $req->data->{mailAddress},
                 $subject, $body, $html
             )
           )
@@ -335,13 +364,20 @@ sub _reset {
 
     # User has a valid mailToken, allow to change password
     # A token is required
-    $self->ott->setToken( $req, { %{ $req->sessionInfo }, pwdAllowed => 1 } );
+    $self->ott->setToken(
+        $req,
+        {
+            %{ $req->sessionInfo },
+            pwdAllowed => $self->conf->{passwordResetAllowedRetries}
+        }
+    );
     return PE_PASSWORDFIRSTACCESS if ( $req->method eq 'GET' );
     return PE_PASSWORDFORMEMPTY;
 }
 
 sub changePwd {
     my ( $self, $req ) = @_;
+    my %tplPrms;
     $self->logger->debug('Change password form response');
 
     if ( my $token = $req->param('token') ) {
@@ -360,7 +396,8 @@ sub changePwd {
         $self->userLogger->error('User tries to change password without token');
         return PE_NOTOKEN;
     }
-    unless ( delete $req->sessionInfo->{pwdAllowed} ) {
+
+    unless ( $req->sessionInfo->{pwdAllowed}-- ) {
         $self->userLogger->error(
             'User tries to use another token to change a password');
         return PE_NOTOKEN;
@@ -377,41 +414,49 @@ sub changePwd {
 
         $self->logger->debug( "Generated password: " . $password );
 
-        $req->datas->{newpassword}     = $password;
-        $req->datas->{confirmpassword} = $password;
-        $req->datas->{forceReset}      = 1;
+        $req->data->{newpassword}     = $password;
+        $req->data->{confirmpassword} = $password;
+        $req->data->{forceReset}      = 1;
+        $tplPrms{RESET}               = 1;
     }
 
     # Else a password is required in request
     else {
-        $req->datas->{newpassword}     = $req->param('newpassword');
-        $req->datas->{confirmpassword} = $req->param('confirmpassword');
-        unless ($req->datas->{newpassword}
-            and $req->datas->{confirmpassword}
-            and $req->datas->{newpassword} eq $req->datas->{confirmpassword} )
+        $req->data->{newpassword}     = $req->param('newpassword');
+        $req->data->{confirmpassword} = $req->param('confirmpassword');
+        unless ($req->data->{newpassword}
+            and $req->data->{confirmpassword}
+            and $req->data->{newpassword} eq $req->data->{confirmpassword} )
         {
             $self->ott->setToken( $req, $req->sessionInfo );
-            return PE_PASSWORDFORMEMPTY;
+            ( $req->data->{newpassword} && $req->data->{confirmpassword} )
+              ? return PE_PASSWORD_MISMATCH
+              : return PE_PASSWORDFORMEMPTY;
+
         }
-
-        # Modify the password TODO: change this
-        # Populate $req->{user} for logging purpose
-        my $tmp = $self->conf->{portalRequireOldPassword};
-        $self->conf->{portalRequireOldPassword} = 0;
-        $req->{user} = $req->{mail};
-        my $result = $self->p->_passwordDB->modifyPassword($req);
-        $req->{user} = undef;
-
-        # Mail token can be used only one time, delete the session if all is ok
-        return $result unless ( $result == PE_PASSWORD_OK or $result == PE_OK );
     }
 
+    # Modify the password TODO: change this
+    # Populate $req->{user} for logging purpose
+    my $tmp = $self->conf->{portalRequireOldPassword};
+    $self->conf->{portalRequireOldPassword} = 0;
+    $req->user( $req->{sessionInfo}->{_user} );
+    my $result =
+      $self->p->_passwordDB->modifyPassword( $req,
+        $req->data->{newpassword}, 1 );
+    $req->{user} = undef;
+    $self->conf->{portalRequireOldPassword} = $tmp;
+
+    # Mail token can be used only one time, delete the session if all is ok
+    return $result unless ( $result == PE_PASSWORD_OK or $result == PE_OK );
+
     # Send mail containing the new password
-    $req->datas->{mailAddress} ||=
+    $req->data->{mailAddress} ||=
       $self->p->getFirstValue(
         $req->{sessionInfo}->{ $self->conf->{mailSessionKey} } );
 
     # Build mail content
+    $tplPrms{MAIN_LOGO} = $self->conf->{portalMainLogo};
     my $tr      = $self->translate($req);
     my $subject = $self->conf->{mailSubject};
     unless ($subject) {
@@ -428,28 +473,22 @@ sub changePwd {
     else {
 
         # Use HTML template
-        my $tplfile =
-            $self->conf->{templateDir} . '/'
-          . $self->conf->{portalSkin}
-          . '/mail_password.tpl';
-        $tplfile = $self->conf->{templateDir} . '/common/mail_password.tpl'
-          unless ( -e $tplfile );
-        my $template = HTML::Template->new(
-            filename => $tplfile,
-            filter   => $tr,
+        $body = $self->loadTemplate(
+            'mail_password',
+            filter => $tr,
+            params => \%tplPrms
         );
-        $body = $template->output();
         $html = 1;
     }
 
     # Replace variables in body
-    my $password = $req->datas->{newpassword};
+    my $password = $req->data->{newpassword};
     $body =~ s/\$password/$password/g;
-    $body =~ s/\$(\w+)/$req->{sessionInfo}->{$1}/ge;
+    $body =~ s/\$(\w+)/$req->{sessionInfo}->{$1} || ''/ge;
 
     # Send mail
     return PE_MAILERROR
-      unless $self->send_mail( $req->datas->{mailAddress}, $subject, $body,
+      unless $self->send_mail( $req->data->{mailAddress}, $subject, $body,
         $html );
 
     PE_MAILOK;
@@ -469,22 +508,23 @@ sub display {
     my ( $self, $req ) = @_;
     $self->logger->debug( 'Display called with code: ' . $req->error );
     my %tplPrm = (
-        SKIN_PATH       => '/static',
+        SKIN_PATH       => $self->conf->{staticPrefix},
         SKIN            => $self->p->getSkin($req),
         SKIN_BG         => $self->conf->{portalSkinBackground},
+        MAIN_LOGO       => $self->conf->{portalMainLogo},
         AUTH_ERROR      => $req->error,
         AUTH_ERROR_TYPE => $req->error_type,
-        CHOICE_PARAM    => $self->conf->{authChoiceParam},
+        AUTH_URL        => $req->data->{_url},
         CHOICE_VALUE    => $req->{_authChoice},
-        EXPMAILDATE     => $req->datas->{expMailDate},
-        EXPMAILTIME     => $req->datas->{expMailTime},
-        STARTMAILDATE   => $req->datas->{startMailDate},
-        STARTMAILTIME   => $req->datas->{startMailTime},
-        MAILALREADYSENT => $req->datas->{mailAlreadySent},
+        EXPMAILDATE     => $req->data->{expMailDate},
+        EXPMAILTIME     => $req->data->{expMailTime},
+        STARTMAILDATE   => $req->data->{startMailDate},
+        STARTMAILTIME   => $req->data->{startMailTime},
+        MAILALREADYSENT => $req->data->{mailAlreadySent},
         MAIL            => (
-            $self->p->checkXSSAttack( 'mail', $req->{mail} )
+            $self->p->checkXSSAttack( 'mail', $req->{user} )
             ? ""
-            : $self->{mail}
+            : $req->{user}
         ),
         DISPLAY_FORM            => 0,
         DISPLAY_RESEND_FORM     => 0,
@@ -492,11 +532,10 @@ sub display {
         DISPLAY_MAILSENT        => 0,
         DISPLAY_PASSWORD_FORM   => 0,
     );
-    if ( $req->datas->{mailToken}
-        and !$self->p->checkXSSAttack( 'mail_token', $req->datas->{mailToken} )
-      )
+    if ( $req->data->{mailToken}
+        and !$self->p->checkXSSAttack( 'mail_token', $req->data->{mailToken} ) )
     {
-        $tplPrm{MAIL_TOKEN} = $req->datas->{mailToken};
+        $tplPrm{MAIL_TOKEN} = $req->data->{mailToken};
     }
 
     # Display captcha if it's enabled
@@ -517,7 +556,7 @@ sub display {
             or $req->error == PE_CAPTCHAERROR
             or $req->error == PE_CAPTCHAEMPTY
         )
-        and !$req->datas->{mailToken}
+        and !$req->data->{mailToken}
       )
     {
         $self->logger->debug('Display form');
@@ -543,7 +582,7 @@ sub display {
     }
 
     # Display password change form
-    elsif ( $req->datas->{mailToken}
+    elsif ( $req->data->{mailToken}
         and $req->error != PE_MAILERROR
         and $req->error != PE_BADMAILTOKEN
         and $req->error != PE_MAILOK )
@@ -552,8 +591,16 @@ sub display {
         $tplPrm{DISPLAY_PASSWORD_FORM} = 1;
     }
 
+    # Display password change form again if passwords mismatch
+    elsif ($req->error == PE_PASSWORDFORMEMPTY
+        || $req->error == PE_PASSWORD_MISMATCH )
+    {
+        $self->logger->debug('Display password form');
+        $tplPrm{DISPLAY_PASSWORD_FORM} = $req->sessionInfo->{pwdAllowed};
+    }
+
     # Custom template parameters
-    if ( my $customParams = $self->p->getCustomTemplateParameters() ) {
+    if ( my $customParams = $self->p->customParameters ) {
         foreach ( keys %$customParams ) {
             $tplPrm{$_} = $customParams->{$_};
         }
