@@ -10,20 +10,23 @@ use Lemonldap::NG::Portal::Main::Constants qw(
   PE_SENDRESPONSE
 );
 
-our $VERSION = '1.9.991_01';
+our $VERSION = '2.0.0';
 
 extends 'Lemonldap::NG::Portal::Main::Plugin';
 
 # INTERFACE
 
-use constant afterDatas => 'newDevice';
+use constant endAuth => 'newDevice';
 
 use constant beforeAuth => 'check';
 
+use constant beforeLogout => 'logout';
+
 # INITIALIZATION
 
 has ott => (
     is      => 'rw',
+    lazy    => 1,
     default => sub {
         my $ott =
           $_[0]->{p}->loadModule('Lemonldap::NG::Portal::Lib::OneTimeToken');
@@ -35,6 +38,7 @@ has ott => (
 # Default timeout: 1 month
 has timeout => (
     is      => 'rw',
+    lazy    => 1,
     default => sub {
         $_[0]->{conf}->{stayConnectedTimeout} || 2678400;
     }
@@ -75,7 +79,7 @@ sub newDevice {
     return PE_OK;
 }
 
-# Store datas in a long-time session
+# Store data in a long-time session
 sub storeBrowser {
     my ( $self, $req ) = @_;
     $req->urldc( $req->param('url') );
@@ -94,15 +98,17 @@ sub storeBrowser {
                             _utime          => time + $self->timeout,
                             _session_uid    => $uid,
                             _connectedSince => time,
-                            dataKeep        => $req->datas->{dataToKeep},
+                            dataKeep        => $req->data->{dataToKeep},
                             fingerprint     => $fg,
                         },
                     );
+
+                    # Cookie available 30 days
                     $req->addCookie(
                         $self->p->cookie(
                             name    => 'llngconnexion',
                             value   => $ps->id,
-                            expires => '+1M',
+                            max_age => 2592000,
                         )
                     );
                 }
@@ -147,7 +153,7 @@ sub check {
                 and my $token = $req->param('token') )
             {
                 if ( my $prm = $self->ott->getToken($token) ) {
-                    $req->datas->{dataKeep} = $ps->data->{dataKeep};
+                    $req->data->{dataKeep} = $ps->data->{dataKeep};
                     $self->logger->debug('Persistent connection found');
                     if ( $fg eq $ps->data->{fingerprint} ) {
                         $req->user($uid);
@@ -192,4 +198,16 @@ sub check {
     return PE_OK;
 }
 
+sub logout {
+    my ( $self, $req ) = @_;
+    $req->addCookie(
+        $self->p->cookie(
+            name    => 'llngconnexion',
+            value   => 0,
+            expires => 'Wed, 21 Oct 2015 00:00:00 GMT'
+        )
+    );
+    PE_OK;
+}
+
 1;
